# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gajim.main import gi_require_versions

gi_require_versions()

from gajim.common import configpaths

configpaths.init()



"""
This type stub file was generated by pyright.
"""

from ._base import Icon as Icon
from ._base import Menu as Menu
from ._base import MenuItem as MenuItem

def backend() -> Icon:
    """Returns the backend module."""
    ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gtk

from gajim.gtk.groupchat_nick_chooser import GroupChatNickChooser
from gajim.gtk.window import GajimAppWindow

from . import util


class TestGroupChatNickChooser(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=700,
            default_height=700,
            add_window_padding=True,
            header_bar=True,
        )

        self._main_box = Gtk.Box(
            hexpand=True, halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER
        )

        self._nick_chooser = GroupChatNickChooser()
        self._nick_chooser.set_text("Fancy Nickname")

        self._main_box.append(self._nick_chooser)

        self.set_child(self._main_box)

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestGroupChatNickChooser()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gtk

from gajim.gtk.assistant import Assistant
from gajim.gtk.assistant import AssistantPage

from . import util


class TestAssistant(Assistant):
    def __init__(self):
        Assistant.__init__(self)

        self.add_button("back", "Back")
        self.add_button("close", "Close", "destructive-action")
        self.add_button("forward", "Forward", "suggested-action", complete=True)

        self.set_button_visible_func(self._visible_func)

        self.add_pages({"start": Start()})

        progress = self.add_default_page("progress")
        progress.set_title("Executing...")
        progress.set_text("Something is in progress...")

        error = self.add_default_page("error")
        error.set_title("Error")
        error.set_heading("Error Heading")
        error.set_text("This is the error text")

        success = self.add_default_page("success")
        success.set_title("Success")
        success.set_heading("Success Heading")
        success.set_text("This is the success text")

        self.connect("button-clicked", self._on_button_clicked)
        self.connect("page-changed", self._on_page_changed)

        self.show_first_page()

    @staticmethod
    def _visible_func(_assistant: Assistant, page_name: str) -> list[str]:
        if page_name == "start":
            return ["forward"]

        if page_name == "progress":
            return ["forward", "back"]

        if page_name == "success":
            return ["forward", "back"]

        if page_name == "error":
            return ["back", "close"]
        raise ValueError("page %s unknown" % page_name)

    def _on_button_clicked(self, _assistant: Assistant, button_name: str) -> None:
        page = self.get_current_page()
        if button_name == "forward":
            if page == "start":
                self.show_page("progress", Gtk.StackTransitionType.SLIDE_LEFT)
            elif page == "progress":
                self.show_page("success", Gtk.StackTransitionType.SLIDE_LEFT)
            elif page == "success":
                self.show_page("error", Gtk.StackTransitionType.SLIDE_LEFT)
            return

        if button_name == "back":
            if page == "progress":
                self.show_page("start")
            if page == "success":
                self.show_page("progress")
            if page == "error":
                self.show_page("success")
            return

        if button_name == "close":
            self.close()

    def _on_page_changed(self, _assistant: Assistant, page_name: str) -> None:
        if page_name == "start":
            self.set_default_button("forward")

        elif page_name == "progress":
            self.set_default_button("forward")

        elif page_name == "success":
            self.set_default_button("forward")

        elif page_name == "error":
            self.set_default_button("back")


class Start(AssistantPage):
    def __init__(self):
        AssistantPage.__init__(self)

        self.title = "Start"
        self.complete = False

        heading = Gtk.Label(label="Test Assistant")
        heading.add_css_class("title-1")

        label1 = Gtk.Label(
            label="This is label 1 with some text",
            wrap=True,
            max_width_chars=50,
            halign=Gtk.Align.CENTER,
            justify=Gtk.Justification.CENTER,
            margin_bottom=24,
        )

        entry = Gtk.Entry()
        entry.set_activates_default(True)
        entry.connect("changed", self._on_changed)

        self._server = Gtk.CheckButton.new_with_mnemonic("A fancy checkbox")
        self._server.set_halign(Gtk.Align.CENTER)

        self.append(heading)
        self.append(label1)
        self.append(entry)
        self.append(self._server)

    def _on_changed(self, entry: Gtk.Entry) -> None:
        self.complete = bool(entry.get_text())
        self.update_page_complete()


util.init_settings()

window = TestAssistant()
window.show()

util.run_app()



from typing import Any

class KeyringBackend:
    def get_password(self, service: str, username: str) -> str | None: ...
    def set_password(self, service: str, username: str, password: str) -> None: ...
    def delete_password(self, service: str, username: str) -> None: ...
    def with_properties(self, **kwargs: Any) -> KeyringBackend: ...

def get_all_keyring() -> list[KeyringBackend]: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gajim.common import app

from gajim.gtk.workspace_dialog import WorkspaceDialog

from . import util

ACCOUNT = "test"

util.init_settings()
app.settings.get_workspace_count = MagicMock(return_value=2)

window = WorkspaceDialog()
window.show()

util.run_app()



#!/usr/bin/env python3

# File needs to stay compatible to Python 3.7
# because its run on the Gajim webserver

import argparse
import re
import subprocess
from pathlib import Path

TRANS_DIR = Path("po")
TRANS_TEMPLATE = TRANS_DIR / "gajim.pot"
REPO_DIR = Path(__file__).resolve().parent.parent


TRANSLATABLE_FILES = [
    "gajim/**/*.py",
    "gajim/**/*.ui",
    "data/org.gajim.Gajim.desktop.in",
    "data/org.gajim.Gajim.metainfo.xml.in",
]


def template_is_equal(old_template_path: Path, new_template: str) -> bool:
    with open(old_template_path, encoding="utf8") as f:
        old_template = f.read()

    pattern = r'"POT-Creation-Date: .*\n"'

    old_template = re.sub(pattern, "", old_template, count=1)
    new_template = re.sub(pattern, "", new_template, count=1)

    return old_template == new_template


def update_translation_template() -> bool:
    paths: list[Path] = []
    for file_path in TRANSLATABLE_FILES:
        paths += list(REPO_DIR.rglob(file_path))

    cmd = [
        "xgettext",
        "-o",
        "-",
        "--add-comments=Translators",
        "--from-code=utf-8",
        "--keyword=p_:1c,2",
        "--no-location",
        "--sort-output",
        "--package-name=Gajim",
    ]

    for path in paths:
        cmd.append(str(path))

    result = subprocess.run(
        cmd, cwd=REPO_DIR, text=True, check=True, capture_output=True
    )

    template = result.stdout

    if TRANS_TEMPLATE.exists() and template_is_equal(TRANS_TEMPLATE, template):
        # No new strings were discovered
        return False

    with open(TRANS_TEMPLATE, "w", encoding="utf8") as f:
        f.write(template)
    return True


def update_translation_files() -> None:
    for file in TRANS_DIR.glob("*.po"):
        subprocess.run(
            ["msgmerge", "-U", "--sort-output", str(file), TRANS_TEMPLATE],
            cwd=REPO_DIR,
            check=True,
        )


def cleanup_translations() -> None:
    for po_file in TRANS_DIR.glob("*.po"):
        subprocess.run(
            ["msgattrib", "--output-file", str(po_file), "--no-obsolete", str(po_file)],
            cwd=REPO_DIR,
            check=True,
        )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Update Translations")
    parser.add_argument("command", choices=["update", "build", "cleanup"])
    args = parser.parse_args()

    if args.command == "cleanup":
        cleanup_translations()

    elif args.command == "update":
        update_translation_template()
        update_translation_files()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Occupant
from gajim.common.storage.archive.models import Retraction
from gajim.common.storage.archive.storage import MessageArchiveStorage
from test.database.util import mk_utc_dt


class RetractionTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_insert_retraction(self) -> None:
        # Test unique index without occupant id

        uuid = get_uuid()
        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id=uuid,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        pk = self._archive.insert_row(retraction_data, ignore_on_conflict=True)
        self.assertNotEqual(pk, -1)

        with self._archive.get_session() as s:
            retraction = s.scalar(select(Retraction).where(Retraction.pk == pk))
        assert retraction is not None

        self.assertEqual(retraction.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(retraction.id, uuid)
        self.assertEqual(retraction.occupant, None)

        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id=uuid,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(1, UTC),
        )

        pk = self._archive.insert_row(retraction_data, ignore_on_conflict=True)
        self.assertEqual(pk, -1)

        # Test unique index with occupant id

        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid1",
            nickname="nickname1",
            updated_at=mk_utc_dt(0),
        )

        uuid = get_uuid()
        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data,
            id=uuid,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        pk = self._archive.insert_row(retraction_data, ignore_on_conflict=True)
        self.assertNotEqual(pk, -1)

        with self._archive.get_session() as s:
            retraction = s.scalar(select(Retraction).where(Retraction.pk == pk))
        assert retraction is not None
        assert retraction.occupant is not None

        self.assertEqual(retraction.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(retraction.id, uuid)
        self.assertEqual(retraction.occupant.id, "occupantid1")

        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data,
            id=uuid,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(1, UTC),
        )

        pk = self._archive.insert_row(retraction_data, ignore_on_conflict=True)
        self.assertEqual(pk, -1)

    def test_retraction_join(self) -> None:
        uuid = get_uuid()
        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid,
            text="message",
        )

        pk = self._archive.insert_object(message_data)

        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="1",
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        self._archive.insert_row(retraction_data, ignore_on_conflict=True)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.retraction is not None
        self.assertEqual(message.retraction.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(message.retraction.id, "1")
        self.assertEqual(message.retraction.occupant, None)

        # Test incorrect direction does not get joined

        uuid = get_uuid()
        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.OUTGOING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="2",
            stanza_id=uuid,
            text="message",
        )

        pk = self._archive.insert_object(message_data)

        retraction_data = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="2",
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        self._archive.insert_row(retraction_data, ignore_on_conflict=True)

        message = self._archive.get_message_with_pk(pk)
        assert message is not None
        self.assertIsNone(message.retraction)

    def test_retractions_join_groupchat(self) -> None:
        occupant_data1 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid1",
            nickname="nickname1",
            updated_at=mk_utc_dt(0),
        )

        occupant_data2 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid2",
            nickname="nickname2",
            updated_at=mk_utc_dt(0),
        )

        uuid1 = get_uuid()
        message_data1 = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid1,
            text="message",
            occupant_=occupant_data1,
        )

        uuid2 = get_uuid()
        message_data2 = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid2,
            text="message",
            occupant_=occupant_data2,
        )

        pk1 = self._archive.insert_object(message_data1)
        pk2 = self._archive.insert_object(message_data2)

        retraction_data1 = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data1,
            id=uuid1,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        retraction_data2 = Retraction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data2,
            id=uuid2,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        self._archive.insert_row(retraction_data1, ignore_on_conflict=True)
        self._archive.insert_row(retraction_data2, ignore_on_conflict=True)

        message1 = self._archive.get_message_with_pk(pk1)
        message2 = self._archive.get_message_with_pk(pk2)

        assert message1 is not None
        assert message1.retraction is not None
        assert message1.retraction.occupant is not None
        self.assertEqual(message1.retraction.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(message1.retraction.id, uuid1)
        self.assertEqual(message1.retraction.occupant.id, "occupantid1")

        assert message2 is not None
        assert message2.retraction is not None
        assert message2.retraction.occupant is not None
        self.assertEqual(message2.retraction.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(message2.retraction.id, uuid2)
        self.assertEqual(message2.retraction.occupant.id, "occupantid2")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

import datetime as dt
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.const import PresenceShow
from nbxmpp.modules.vcard4 import VCard
from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.const import AvatarSize
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import ContactSettings

from gajim.gtk.avatar import generate_default_avatar
from gajim.gtk.contact_popover import ContactPopover
from gajim.gtk.util.misc import convert_surface_to_texture
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "account"


class TestContactPopover(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )
        contact = self._get_contact()

        scale = self.get_scale_factor()
        image = Gtk.Image.new_from_paintable(
            contact.get_avatar(AvatarSize.ACCOUNT_PAGE, scale)
        )
        image.set_pixel_size(AvatarSize.ACCOUNT_PAGE)

        button = Gtk.MenuButton(halign=Gtk.Align.CENTER, valign=Gtk.Align.START)
        button.set_child(image)
        button.add_css_class("flat")
        self.set_child(button)

        popover = ContactPopover(contact)

        button.set_popover(popover)

        popover._on_vcard_received(  # pyright: ignore
            JID.from_string("test@test.com"), self._get_vcard()
        )

    def _get_contact(self) -> BareContact:
        contact = MagicMock(spec="BareContact")
        contact.connect = MagicMock()
        contact.disconnect_all_from_obj = MagicMock()

        contact.account = ACCOUNT
        contact.is_groupchat = False
        contact.is_self = False
        avatar = convert_surface_to_texture(
            generate_default_avatar(
                "T", (0.2, 0.1, 0.7), AvatarSize.ACCOUNT_PAGE, self.get_scale_factor()
            )
        )
        contact.get_avatar = MagicMock(return_value=avatar)
        contact.settings = ContactSettings(ACCOUNT, JID.from_string(ACCOUNT))

        contact.name = "Contact Name"
        contact.jid = JID.from_string("contact@example.org")

        contact.show = PresenceShow.ONLINE
        contact.status = "Status Message Text with a really long status message of many word with many characters"  # noqa: E501

        contact.idle_datetime = dt.datetime.now() - dt.timedelta(minutes=15)

        contact.subscription = "from"

        return contact

    def _get_vcard(self) -> VCard:
        vcard = VCard()
        vcard.add_property("org", values=["Gajim"])
        vcard.add_property("role", value="Admin")
        vcard.add_property("tel", value="+1454545544", value_type="text")
        vcard.add_property("email", value="mail@mail.com")
        vcard.add_property("tz", value="Europe/London", value_type="text")
        return vcard

    def _cleanup(self) -> None:
        pass


def get_color(selector: str, *args: Any) -> str:
    if selector == ".gajim-status-online":
        return "rgb(102, 191, 16)"
    if selector == ".gajim-status-away":
        return "rgb(255, 133, 51)"
    return "rgb(100, 100, 100)"


app.get_client = MagicMock()
app.window = MagicMock()

util.init_settings()

app.settings.add_account(ACCOUNT)
app.settings.set_account_setting(ACCOUNT, "address", "user@example.org")

app.css_config = MagicMock()
app.css_config.get_value = MagicMock(side_effect=get_color)

app.plugin_manager = MagicMock()

window = TestContactPopover()
window.show()

util.run_app()



from __future__ import annotations

from pathlib import Path

from . import css as css
from . import serialize as serialize
from . import stylesheets as stylesheets
from .serialize import CSSSerializer
from .stylesheets import MediaList

ser: CSSSerializer

def parseFile(
    filename: str | Path,
    href: str | None = ...,
    media: MediaList | list[str] | str | None = ...,
    title: str | None = ...,
    validate: bool | None = ...,
) -> None: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Iterator
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import ResourceContact
from gajim.common.storage.cache import CacheStorage

from gajim.gtk.resource_selector import ResourceSelector
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
FROM_JID = "test@example.org"


class TestResourceSelector(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            width_request=500,
            height_request=700,
        )
        self.set_child(box)

        contact = self._get_contact()

        self._resources = self._generate_resources()

        self._resource_selector = ResourceSelector(contact)
        self._resource_selector.set_hexpand(True)
        self._resource_selector.connect("selection-changed", self._on_selection_changed)
        box.append(self._resource_selector)

    def _on_selection_changed(
        self, _resource_selector: ResourceSelector, state: bool
    ) -> None:
        print("Selection available:", state)

    def _get_contact(self) -> BareContact:
        contact = MagicMock(spec_set=BareContact)
        contact.account = ACCOUNT
        contact.jid = JID.from_string(FROM_JID)
        contact.name = "Test Contact"
        contact.is_groupchat = False
        contact.iter_resources = MagicMock(side_effect=self._iter_resources)
        return contact

    def _generate_resources(self) -> list[ResourceContact]:
        resources: list[ResourceContact] = []
        for index in range(5):
            resource_contact = MagicMock(spec_set=ResourceContact)
            resource_contact.jid = JID.from_string(f"{FROM_JID}/Resource.{index}")
            resources.append(resource_contact)

        return resources

    def _iter_resources(self) -> Iterator[ResourceContact]:
        yield from self._resources

    def _cleanup(self) -> None:
        pass


util.init_settings()

app.storage.cache = CacheStorage(in_memory=True)
app.storage.cache.init()

window = TestResourceSelector()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import typing

from unittest.mock import MagicMock

from gajim.common import app

from gajim.gtk.groupchat_creation import CreateGroupchatWindow

from . import util

accounts: list[list[str]] = []
for i in range(2):
    accounts.append(
        [f"testacc{i}", f"Account {i}"],
    )

util.init_settings()

app.get_enabled_accounts_with_labels = MagicMock(return_value=accounts)
app.get_number_of_connected_accounts = MagicMock(return_value=2)
app.account_is_available = MagicMock(return_value=True)


class TestModule(MagicMock):
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None:
        super().__init__()
        self.service_jid = "conference.test.example"


class TestClient(MagicMock):
    def __init__(self):  # pyright: ignore
        super().__init__()

    def get_module(self) -> TestModule:
        return TestModule()


app.get_client = MagicMock(return_value=TestClient)


window = CreateGroupchatWindow(accounts[0][0])
window.show()

util.run_app()



"""
This type stub file was generated by pyright.
"""

import typing

from gi.repository import Gtk
from pystray import _base

if not Gtk.init_check()[0]: ...

def mainloop(f: typing.Any) -> typing.Any:
    """Marks a function to be executed in the main loop.

    The function will be scheduled to be executed later in the mainloop.

    :param callable f: The function to execute. Its return value is discarded.
    """
    ...

class GtkIcon(_base.Icon):
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...



"""
This type stub file was generated by pyright.
"""

import typing

import contextlib

from . import _base

display: typing.Any = ...

class XError(Exception):
    """An error that is thrown at the end of a code block managed by a
    :func:`display_manager` if an *X* error occurred.
    """

    ...

@contextlib.contextmanager
def display_manager(display: typing.Any) -> typing.Any:
    """Traps *X* errors and raises an :class:`XError` at the end if any
    error occurred.

    This handler also ensures that the :class:`Xlib.display.Display` being
    managed is sync'd.

    :param Xlib.display.Display display: The *X* display.
    """
    ...

class Icon(_base.Icon):
    _XEMBED_VERSION: typing.Any = ...
    _XEMBED_MAPPED: typing.Any = ...
    _SYSTEM_TRAY_REQUEST_DOCK: typing.Any = ...
    HAS_MENU: bool = ...
    HAS_MENU_RADIO: bool = ...
    HAS_NOTIFICATION: bool = ...
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
    def __del__(self) -> None: ...



#!/usr/bin/env python3

import subprocess
from pathlib import Path

if __name__ == "__main__":
    # Protect the entry point of the application because we use
    # the multiprocessing module with "spawn"

    import gajim
    import gajim.main

    try:
        res = subprocess.check_output(
            [
                "git",
                "-C",
                f"{Path(__file__).parent}",
                "rev-parse",
                "--short=12",
                "HEAD",
            ],
            stderr=subprocess.DEVNULL,
        )
        gajim.__version__ += f"+{res.decode().strip()}"
    except Exception:
        pass

    gajim.main.run()



"""
This type stub file was generated by pyright.
"""

from typing import Optional
from typing import TypeVar
from typing import Union

import datetime

from winrt.windows.data.xml.dom import IXmlNode
from winrt.windows.data.xml.dom import XmlDocument
from winrt.windows.data.xml.dom import XmlElement

from .toast import Toast
from .toast_audio import ToastAudio
from .wrappers import ToastButton
from .wrappers import ToastDisplayImage
from .wrappers import ToastDuration
from .wrappers import ToastInputSelectionBox
from .wrappers import ToastInputTextBox
from .wrappers import ToastProgressBar
from .wrappers import ToastScenario
from .wrappers import ToastSystemButton

IXmlType = TypeVar("IXmlType", IXmlNode, XmlElement)

class ToastDocument:
    """
    The XmlDocument wrapper for toasts, which applies all the
    attributes configured in :class:`~windows_toasts.toast.Toast`
    """

    xmlDocument: XmlDocument
    bindingNode: IXmlType
    _inputFields: int
    def __init__(self, toast: Toast) -> None: ...
    @staticmethod
    def GetAttributeValue(nodeAttribute: IXmlType, attributeName: str) -> str:
        """
        Helper function that returns an attribute's value

        :param nodeAttribute: Node that has the attribute
        :type nodeAttribute: IXmlType
        :param attributeName: Name of the attribute, e.g. "duration"
        :type attributeName: str
        :return: The value of the attribute
        :rtype: str
        """
        ...

    def GetElementByTagName(self, tagName: str) -> Optional[IXmlType]:
        """
        Helper function to get the first element by its tag name

        :param tagName: The name of the tag for the element
        :type tagName: str
        :rtype: IXmlType
        """
        ...

    def SetAttribute(
        self, nodeAttribute: IXmlType, attributeName: str, attributeValue: str
    ) -> None:
        """
        Helper function to set an attribute to a node. <nodeAttribute attributeName="attributeValue" />

        :param nodeAttribute: Node to apply attributes to
        :type nodeAttribute: IXmlType
        :param attributeName: Name of the attribute, e.g. "duration"
        :type attributeName: str
        :param attributeValue: Value of the attribute, e.g. "long"
        :type attributeValue: str
        """
        ...

    def SetNodeStringValue(self, targetNode: IXmlType, newValue: str) -> None:
        """
        Helper function to set the inner value of a node. <text>newValue</text>

        :param targetNode: Node to apply attributes to
        :type targetNode: IXmlType
        :param newValue: Inner text of the node, e.g. "Hello, World!"
        :type newValue: str
        """
        ...

    def SetAttributionText(self, attributionText: str) -> None:
        """
        Set attribution text for the toast. This is used if we're using
        :class:`~windows_toasts.toasters.InteractableWindowsToaster` but haven't set up our own AUMID.
        `AttributionText on Microsoft.com <https://learn.microsoft.com/windows/apps/design/shell/tiles-and
        -notifications/adaptive-interactive-toasts#attribution-text>`_

        :param attributionText: Attribution text to set
        """
        ...

    def SetAudioAttributes(self, audioConfiguration: ToastAudio) -> None:
        """
        Apply audio attributes for the toast. If a loop is requested, the toast duration has to be set to long. `Audio
        on Microsoft.com <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications/adaptive
        -interactive-toasts#audio>`_
        """
        ...

    def SetTextField(self, nodePosition: int) -> None:
        """
        Set a simple text field. `Text elements on Microsoft.com
        <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications/adaptive-interactive-toasts#text-elements>`_

        :param nodePosition: Index of the text fields of the toast type for the text to be written in
        """
        ...

    def SetTextFieldStatic(self, nodePosition: int, newValue: str) -> None:
        """
        :meth:`SetTextField` but static, generally used for scheduled toasts

        :param nodePosition: Index of the text fields of the toast type for the text to be written in
        :param newValue: Content value of the text field
        """
        ...

    def SetCustomTimestamp(self, customTimestamp: datetime.datetime) -> None:
        """
        Apply a custom timestamp to display on the toast and in the notification center. `Custom timestamp on
        Microsoft.com <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications/adaptive
        -interactive-toasts?tabs=xml#custom-timestamp>`_

        :param customTimestamp: The target datetime
        :type customTimestamp: datetime.datetime
        """
        ...

    def AddImage(self, displayImage: ToastDisplayImage) -> None:
        """
        Add an image to display. `Inline image on Microsoft.com <https://learn.microsoft.com/windows/
        apps/design/shell/tiles-and-notifications/adaptive-interactive-toasts#inline-image>`_

        :type displayImage: ToastDisplayImage
        """
        ...

    def SetScenario(self, scenario: ToastScenario) -> None:
        """
        Set whether the notification should be marked as important. `Important Notifications on Microsoft.com
        <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications/adaptive-interactive-toasts
        #important-notifications>`_

        :param scenario: Scenario to mark the toast as
        :type scenario: ToastScenario
        """
        ...

    def AddInput(
        self, toastInput: Union[ToastInputTextBox, ToastInputSelectionBox]
    ) -> None:
        """
        Add a field for the user to input. `Inputs with button bar on Microsoft.com
        <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications/adaptive-interactive
        -toasts#quick-reply-text-box>`_

        :type toastInput: Union[ToastInputTextBox, ToastInputSelectionBox]
        """
        ...

    def SetDuration(self, duration: ToastDuration) -> None:
        """
        Set the duration of the toast. If looping audio is enabled, it will automatically be set to long

        :type duration: ToastDuration
        """
        ...

    def AddAction(self, action: Union[ToastButton, ToastSystemButton]) -> None:
        """
        Adds a button to the toast. Only works on :obj:`~windows_toasts.toasters.InteractableWindowsToaster`

        :type action: Union[ToastButton, ToastSystemButton]
        """
        ...

    def AddProgressBar(self) -> None:
        """
        Add a progress bar on your app notification to keep the user informed of the progress of operations.
        `Progress bar on Microsoft.com <https://learn.microsoft.com/windows/apps/design/shell/tiles-and-notifications
        /adaptive-interactive-toasts#progress-bar>`_
        """
        ...

    def AddStaticProgressBar(self, progressBar: ToastProgressBar) -> None:
        """
        :meth:`AddProgressBar` but static, generally used for scheduled toasts
        """
        ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

from gajim.common import app
from gajim.common.modules.contacts import GroupchatContact
from gajim.common.modules.contacts import GroupchatParticipant

from gajim.gtk.completion.nickname import NicknameCompletionProvider


class Test(unittest.TestCase):
    def test_generate_suggestions(self):
        def _get_mock_participant(nick: str) -> GroupchatParticipant:
            participant = MagicMock(spec_set=GroupchatParticipant)
            participant.name = nick
            return participant

        def _get_resource(nick: str) -> GroupchatParticipant:
            return _get_mock_participant(nick)

        participant_names = [
            "Hugo",
            "Herbert",
            "Robert",
            "Daisy",
            "xavier",
            "7user",
        ]

        participants: list[GroupchatParticipant] = []
        for name in participant_names:
            participants.append(_get_mock_participant(name))

        gen = NicknameCompletionProvider()
        groupchat_contact = MagicMock(spec_set=GroupchatContact)
        groupchat_contact.get_participants = MagicMock(return_value=participants)
        groupchat_contact.get_resource = MagicMock(side_effect=_get_resource)

        results = gen._generate_suggestions(groupchat_contact)  # type: ignore
        self.assertEqual(
            [result.name for result in results],
            ["Harry", "Joe", "7user", "Daisy", "Herbert", "Hugo", "Robert", "xavier"],
        )


app.get_client = MagicMock()

app.storage.archive = MagicMock()
app.storage.archive.get_recent_muc_nicks = MagicMock(return_value=["Harry", "Joe"])

if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common.util.muc import format_private_group_name

from gajim.gtk.util.misc import get_first_grapheme


class Test(unittest.TestCase):
    def disabled_test_get_first_grapheme(self):
        # This test fails on some machines

        self.assertEqual(get_first_grapheme(""), "", "<empty string>")
        self.assertEqual(get_first_grapheme("a"), "a", "a")
        self.assertEqual(get_first_grapheme("ab"), "a", "ab -> a")

        ueber = "u\u0308ber"
        self.assertEqual(get_first_grapheme(ueber), "u\u0308", ueber + " -> ü")

        woman = "\U0001f469"
        zwj = "\u200d"
        vs16 = "\ufe0f"
        fitz4 = "\U0001f3fd"

        farmeress = f"{woman}{zwj}\U0001f33e{vs16}"
        self.assertEqual(get_first_grapheme(farmeress), farmeress, "👩‍🌾️")

        longass = f"{woman}{fitz4}{zwj}\u2764{vs16}{zwj}\U0001f468{fitz4}"
        self.assertEqual(get_first_grapheme(longass), longass, "👩🏽‍❤️‍👨🏽")

        # The following are from
        # https://www.unicode.org/reports/tr29/#Table_Sample_Grapheme_Clusters

        hangul_gag = "\u1100\u1161\u11a8"
        self.assertEqual(get_first_grapheme(hangul_gag), hangul_gag, "각")

        tamil_ni = "\u0ba8\u0bbf"
        self.assertEqual(get_first_grapheme(tamil_ni), tamil_ni, "நி")

        # Fails 🤷 (returns the first char)
        # thai_kam = '\u0E01\u0E33'
        # self.assertEqual(
        #    get_first_grapheme(thai_kam), thai_kam, 'กำ')

        devanagari_ssi = "\u0937\u093f"
        self.assertEqual(get_first_grapheme(devanagari_ssi), devanagari_ssi, "षि")

        # Only in some locales (e.g., Slovak):
        # self.assertEqual(
        #    get_first_grapheme('ch'), 'ch', 'ch -> ch')
        # Actually, Gtk.TextIter.forward_cursor_position() doesn't seem to use
        # tailored algorithms anyway, so even with LANG=sk_SK.UTF-8 this
        # returns 'c', not 'ch'.

        # In most locales (say, any western one):
        devanagari_kshi = "\u0915\u094d" + devanagari_ssi
        self.assertEqual(
            get_first_grapheme(devanagari_kshi), "\u0915\u094d", "क्षि -> क् "
        )
        # This probably won't fail on *any* locale, ever, again because the
        # implementation doesn't seem locale-specific.

    def test_format_private_muc_name(self) -> None:
        self.assertEqual(format_private_group_name(["a"], 5), "a and me")
        self.assertEqual(format_private_group_name(["a", "b"], 5), "a, b")
        self.assertEqual(
            format_private_group_name(["b", "a", "c", "d", "e", "f"], 5),
            "a, b, c, d, e, f",
        )
        self.assertEqual(
            format_private_group_name(["a", "c", "d", "e", "f", "g", "b"], 5),
            "a, b, c, d, e, 2 others",
        )


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from typing import Annotated
from typing import Any
from typing import Generic
from typing import ItemsView
from typing import Iterable
from typing import Iterator
from typing import KeysView
from typing import List
from typing import overload
from typing import SupportsIndex
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union
from typing import ValuesView

import array
import types
from uuid import UUID

MTA: int
STA: int

def init_apartment(apartment_type: int) -> None: ...
def uninit_apartment() -> None: ...
def initialize_with_window(obj: Object, hwnd: int) -> None: ...
def _add_dll_directory(path: str) -> int: ...
def _remove_dll_directory(cookie: int) -> None: ...

_Self = TypeVar("_Self")
_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)  # Any type covariant containers.
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_VT_co = TypeVar("_VT_co", covariant=True)  # Value type covariant containers.

# these classes don't actually exist but are just used to simplify type checking

class Sequence(Generic[_T_co]):
    # collections.abc.Sequence mixin methods
    def index(self, value: Any, start: int = 0, stop: int = ...) -> int: ...
    def count(self, value: Any) -> int: ...
    def __reversed__(self) -> Iterator[_T_co]: ...

class MutableSequence(Sequence[_T]):
    # alias for insert_at
    def insert(self, index: SupportsIndex, value: _T) -> None: ...
    # collections.abc.MutableSequence mixin methods
    def extend(self, values: Iterable[_T]) -> None: ...
    def reverse(self) -> None: ...
    def pop(self, index: int = -1) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def __iadd__(self: _Self, values: Iterable[_T]) -> _Self: ...

class Mapping(Generic[_KT, _VT_co]):
    # collections.abc.Mapping mixin methods
    @overload
    def get(self, __key: _KT) -> _VT_co | None: ...
    @overload
    def get(self, __key: _KT, default: _VT_co | _T) -> _VT_co | _T: ...
    def items(self) -> ItemsView[_KT, _VT_co]: ...
    def keys(self) -> KeysView[_KT]: ...
    def values(self) -> ValuesView[_VT_co]: ...
    def __eq__(self, __other: object) -> bool: ...

class MutableMapping(Mapping[_KT, _VT]):
    # collections.abc.MutableMapping mixin methods
    @overload
    def pop(self, __key: _KT) -> _VT: ...
    @overload
    def pop(self, __key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, __key: _KT, default: _T) -> _VT | _T: ...
    def popitem(self) -> Tuple[_KT, _VT]: ...
    @overload
    def setdefault(
        self: MutableMapping[_KT, _T | None], __key: _KT, __default: None = None
    ) -> _T | None: ...
    @overload
    def setdefault(self, __key: _KT, __default: _VT) -> _VT: ...
    @overload
    def update(self, __m: Iterable[Tuple[_KT, _VT]], **kwargs: _VT) -> None: ...
    @overload
    def update(self, **kwargs: _VT) -> None: ...

# actual runtime classes

class Object:
    @property
    def _iids_(self) -> Array[UUID]: ...
    @property
    def _runtime_class_name_(self) -> str: ...

class Array(MutableSequence[_T]):
    _winrt_element_type_name_: str
    def __class_getitem__(cls, key: Any) -> types.GenericAlias: ...
    @overload
    def __init__(
        self,
        type: Union[Type[_T], str],
        size: int,
    ) -> None: ...
    @overload
    def __init__(
        self,
        type: Union[Type[_T], str],
        initializer: memoryview,
    ) -> None: ...
    @overload
    def __init__(self, type: str, initializer: array.array[str]) -> None: ...
    @overload
    def __init__(
        self,
        type: Union[Type[_T], str],
        initializer: Array[_T],
    ) -> None: ...
    @overload
    def __init__(
        self,
        type: Union[Type[_T], str],
        initializer: List[_T],
        /,
    ) -> None: ...
    @overload
    def __init__(
        self,
        __type: Union[Type[_T], str],
        __initializer: Tuple[_T],
    ) -> None: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self, index: SupportsIndex) -> _T: ...
    @overload
    def __getitem__(self, index: slice) -> MutableSequence[_T]: ...
    @overload
    def __setitem__(self, index: SupportsIndex, value: _T) -> None: ...
    @overload
    def __setitem__(self, index: slice, value: Iterable[_T]) -> None: ...
    @overload
    def __delitem__(self, index: SupportsIndex) -> None: ...
    @overload
    def __delitem__(self, index: slice) -> None: ...

# type aliases with struct format string annotation
Int8 = Annotated[int, "b"]
UInt8 = Annotated[int, "B"]
Int16 = Annotated[int, "h"]
UInt16 = Annotated[int, "H"]
Int32 = Annotated[int, "i"]
UInt32 = Annotated[int, "I"]
Int64 = Annotated[int, "q"]
UInt64 = Annotated[int, "Q"]
Single = Annotated[float, "f"]
Double = Annotated[float, "d"]
Char16 = Annotated[str, "u"]



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from gi.repository import Gtk

from gajim.gtk.dropdown import GajimDropDown
from gajim.gtk.window import GajimAppWindow

from . import util


class TestGajimDropDown(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(box)

        gajim_drop_down1: GajimDropDown[str] = GajimDropDown(fixed_width=20)
        gajim_drop_down1.set_data(
            {
                "key1": "Test 10000",
                "key2": "Test 2 Very Very Very Very Very Long Key",
                "key3": "Test 3",
                "Key": "Value",
                "Another Key": "Another Value",
            }
        )
        gajim_drop_down1.set_enable_search(True)
        gajim_drop_down1.connect("notify::selected", self._on_item_selected)
        box.append(gajim_drop_down1)

        gajim_drop_down2: GajimDropDown[str] = GajimDropDown(
            data=["key1", "key2", "key3"]
        )
        gajim_drop_down2.set_selected(2)
        gajim_drop_down2.connect("notify::selected", self._on_item_selected)
        box.append(gajim_drop_down2)

    def _on_item_selected(self, drop_down: GajimDropDown[str], *args: Any) -> None:
        print("Index:", drop_down.get_selected())
        item = drop_down.get_selected_item()
        if item is not None:
            print("Item key:", item.get_property("key"))
            print("Item value:", item.get_property("value"))

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestGajimDropDown()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

import gi

gi.require_version("Gst", "1.0")

from gi.repository import Gst
from gi.repository import Gtk

from gajim.common import app

from gajim.gtk.gstreamer import create_video_elements
from gajim.gtk.window import GajimAppWindow

from . import util


class TestGstreamer(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True)
        self.set_child(box)

        video_elements = create_video_elements()
        if video_elements is None:
            return

        sink, paintable, name = video_elements
        print(sink, paintable, name)

        pipeline = Gst.Pipeline.new()
        pipeline.add(sink)

        if name == "gtkglsink":
            src = Gst.ElementFactory.make("gltestsrc")
        else:
            src = Gst.ElementFactory.make("videotestsrc")

        if src is None:
            return

        pipeline.add(src)
        src.link(sink)
        pipeline.set_state(Gst.State.PLAYING)

        picture = Gtk.Picture(hexpand=True, vexpand=True, paintable=paintable)
        box.append(picture)

    def _cleanup(self) -> None:
        pass


Gst.init()

app.is_installed = MagicMock(return_value=True)

util.init_settings()

window = TestGstreamer()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

import re
import unittest

from gajim.common import regex

TEST_URIS = [
    "scheme:",
    "http:///",
    "http://",
    "http://google.com/",
    "http://google.com",
    "http://www.google.ca/search?q=xmpp",
    "http://tools.ietf.org/html/draft-saintandre-rfc3920bis-05#section-12.3",
    "http://en.wikipedia.org/wiki/Protocol_(computing)",
    "http://en.wikipedia.org/wiki/Protocol_%28computing%29",
    "mailto:test@example.org",
    "xmpp:example-node@example.com",
    "xmpp:example-node@example.com/some-resource",
    "xmpp:example-node@example.com?message",
]

JIDS = [
    "example-node@example.com",
    "me@[::1]",
    "myself@127.13.42.69",
    # 'myself@127.13.42.69/localhost',  TODO
    "#room%irc.example@biboumi.xmpp.example",
    r"here\27s_a_wild_\26_\2fcr%zy\2f_address_for\3a\3cwv\3e(\22IMPS\22)@example.com",
    r"CN=D\27Artagnan\20Saint-Andr\E9,O=Example\20\26\20Company,\20Inc.,DC=example,DC=com@st.example.com",
    "δοκιμή@παράδειγμα.δοκιμή",
]


class Test(unittest.TestCase):
    def test_links_regexp_entire(self):
        def assert_matches_all(regex: Any, uri_str: str) -> None:
            m = regex.match(uri_str)
            if m is None:
                raise AssertionError("Failed: %s" % uri_str)

            str_span = (0, len(uri_str))
            self.assertEqual(m.span(), str_span, uri_str)

        for uri in TEST_URIS:
            assert_matches_all(re.compile(regex.IRI), uri)

        for addr in JIDS:
            assert_matches_all(re.compile(regex.XMPP.jid), addr)


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gi.repository import Gdk
from gi.repository import Gtk

from gajim.common import app

from gajim.gtk.avatar import make_workspace_avatar
from gajim.gtk.const import DEFAULT_WORKSPACE_COLOR
from gajim.gtk.util.styling import make_rgba
from gajim.gtk.util.styling import rgba_to_float
from gajim.gtk.window import GajimAppWindow
from gajim.gtk.workspace_listbox import WorkspaceListBox

from . import util


class TestWorkspaceListBox(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        self._box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(self._box)

        chat_page = MagicMock()
        sidebar = WorkspaceListBox()
        sidebar.set_chat_page(chat_page)
        self._box.append(sidebar)

        app.app = MagicMock()
        app.app.avatar_storage.get_workspace_texture.side_effect = (
            self._get_workspace_texture
        )

        for workspace_id in app.settings.get_workspaces():
            sidebar.add_workspace(workspace_id)

    def _get_workspace_texture(
        self, workspace_id: str, size: int, scale: int
    ) -> Gdk.Texture | None:
        name = app.settings.get_workspace_setting(workspace_id, "name")
        rgba = make_rgba(DEFAULT_WORKSPACE_COLOR)
        return make_workspace_avatar(name, rgba_to_float(rgba), size, scale)

    def _cleanup(self) -> None:
        pass


app.window = MagicMock()

util.init_settings()

for i in range(10):
    app.settings.add_workspace(str(i))


window = TestWorkspaceListBox()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import datetime as dt
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID

import gajim.common.storage.archive.models as mod
from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.modules.chat_markers import DisplayedMarkerData
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.util.datetime import utc_now

from gajim.gtk.avatar import AvatarStorage
from gajim.gtk.conversation.avatar_stack import AvatarStack
from gajim.gtk.css_config import CSSConfig
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
REMOTE_JID = JID.from_string("user@domain.org")


class TestAvatarStack(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        app.app = MagicMock()
        app.app.avatar_storage = AvatarStorage()

        box = Gtk.Box(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True)
        self.set_child(box)

        avatar_stack = AvatarStack(ACCOUNT)
        box.append(avatar_stack)

        markers = app.storage.archive.get_last_display_markers(
            "testacc1", JID.from_string("user@domain.org")
        )
        markers = [DisplayedMarkerData.from_model(ACCOUNT, m) for m in markers]
        avatar_stack.set_data(markers)

    def _cleanup(self) -> None:
        pass


def insert_test_markers() -> None:
    occupants: dict[int, mod.Occupant] = {}
    for i in range(10):
        occupants[i] = mod.Occupant(
            account_=ACCOUNT,
            remote_jid_=REMOTE_JID,
            id=f"occupantid{i}",
            nickname=f"nickname{i}",
            updated_at=utc_now(),
        )

    uuid = get_uuid()

    for i in range(10):
        marker = mod.DisplayedMarker(
            account_=ACCOUNT,
            remote_jid_=REMOTE_JID,
            occupant_=occupants[i],
            id=uuid,
            timestamp=utc_now() - dt.timedelta(seconds=100 * i),
        )

        app.storage.archive.insert_object(marker)


util.init_settings()

app.settings.add_account(ACCOUNT)
app.settings.set_account_setting("testacc1", "address", "user@domain.org")
app.storage.archive = MessageArchiveStorage(in_memory=True)
app.storage.archive.init()
insert_test_markers()

app.css_config = CSSConfig()

app.get_client = MagicMock()

window = TestAvatarStack()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import tempfile
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common import configpaths
from gajim.common.const import AvatarSize
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import ContactSettings
from gajim.common.storage.archive import models as mod
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.storage.events.storage import EventStorage
from gajim.common.util.datetime import utc_now

from gajim.gtk.avatar import generate_default_avatar
from gajim.gtk.control import ChatControl
from gajim.gtk.util.misc import convert_surface_to_texture
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
FROM_JID = "contact@test.tld"
BASE_TIMESTAMP = 1672531200


class TestConversationView(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        self._chat_control = ChatControl()
        app.settings.set("hide_groupchat_occupants_list", True)

        contact = self._get_contact()
        self._chat_control.switch_contact(contact)

        jump_to_button = Gtk.Button(label="Jump to 500")
        jump_to_button.connect("clicked", self._on_jump_to_clicked)
        button_box = Gtk.Box(spacing=6, halign=Gtk.Align.CENTER, margin_bottom=6)
        button_box.append(jump_to_button)

        box = Gtk.Box(spacing=6, orientation=Gtk.Orientation.VERTICAL)
        box.append(self._chat_control.get_widget())
        box.append(button_box)
        self.set_child(box)

    def _get_contact(self) -> BareContact:
        contact = MagicMock(spec="BareContact")
        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = JID.from_string(FROM_JID)
        contact.name = "Test Contact"
        contact.is_groupchat = False
        avatar = convert_surface_to_texture(
            generate_default_avatar("T", (0.2, 0.1, 0.7), AvatarSize.ROSTER, 1)
        )
        contact.get_avatar = MagicMock(return_value=avatar)
        contact.settings = ContactSettings(ACCOUNT, JID.from_string(ACCOUNT))
        contact.disconnect_all_from_obj = MagicMock()
        return contact

    def _on_jump_to_clicked(self, _button: Gtk.Button) -> None:
        # BASE_TIMESTAMP + 500
        self._chat_control.scroll_to_message(500, utc_now())

    def _cleanup(self) -> None:
        pass


def add_archive_messages() -> None:
    remote_jid = JID.from_string(FROM_JID)
    timestamp = BASE_TIMESTAMP
    for num in range(1000):
        message_data = mod.Message(
            account_=ACCOUNT,
            remote_jid_=remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            resource=None,
            text=str(num),
            id=str(num),
            stanza_id=str(num),
        )

        app.storage.archive.insert_object(message_data)

        timestamp += 1


app.get_client = MagicMock()
app.window = MagicMock()

util.init_settings()

app.settings.add_account(ACCOUNT)
app.settings.set_account_setting(ACCOUNT, "address", "user@domain.org")

app.storage.events = EventStorage()
app.storage.events.init()

configpaths.set_separation(True)
configpaths.set_config_root(tempfile.gettempdir())
configpaths.init()

app.storage.archive = MessageArchiveStorage(in_memory=True)
app.storage.archive.init()
add_archive_messages()

window = TestConversationView()
window.show()

util.run_app()



"""
This type stub file was generated by pyright.
"""

from typing import Callable
from typing import Literal
from typing import Optional
from typing import TypeVar
from typing import Union

import datetime
from collections.abc import Iterable

from winrt.windows.ui.notifications import ToastDismissedEventArgs
from winrt.windows.ui.notifications import ToastFailedEventArgs

from .events import ToastActivatedEventArgs
from .toast_audio import ToastAudio
from .wrappers import ToastButton
from .wrappers import ToastDisplayImage
from .wrappers import ToastDuration
from .wrappers import ToastInputSelectionBox
from .wrappers import ToastInputTextBox
from .wrappers import ToastProgressBar
from .wrappers import ToastScenario
from .wrappers import ToastSystemButton

ToastInput = TypeVar("ToastInput", ToastInputTextBox, ToastInputSelectionBox)

class Toast:
    audio: Optional[ToastAudio]
    duration: Literal[ToastDuration.Default, ToastDuration.Long, ToastDuration.Short]
    expiration_time: Optional[datetime.datetime]
    group: Optional[str]
    scenario: ToastScenario
    suppress_popup: bool
    timestamp: Optional[datetime.datetime]
    progress_bar: Optional[ToastProgressBar]
    attribution_text: Optional[str]
    on_activated: Optional[Callable[[ToastActivatedEventArgs], None]]
    on_dismissed: Optional[Callable[[ToastDismissedEventArgs], None]]
    on_failed: Optional[Callable[[ToastFailedEventArgs], None]]
    actions: list[Union[ToastButton, ToastSystemButton]]
    images: list[ToastDisplayImage]
    inputs: list[ToastInput]
    text_fields: list[Optional[str]]
    tag: str
    updates: int
    _launch_action: Optional[str]
    def __init__(
        self,
        text_fields: Union[
            list[Optional[str]], tuple[Optional[str]], set[Optional[str]]
        ] = ...,
        audio: Optional[ToastAudio] = ...,
        duration: ToastDuration = ...,
        expiration_time: Optional[datetime.datetime] = ...,
        group: Optional[str] = ...,
        launch_action: Optional[str] = ...,
        progress_bar: Optional[ToastProgressBar] = ...,
        attribution_text: Optional[str] = ...,
        scenario: ToastScenario = ...,
        suppress_popup: bool = ...,
        timestamp: Optional[datetime.datetime] = ...,
        on_activated: Optional[Callable[[ToastActivatedEventArgs], None]] = ...,
        on_dismissed: Optional[Callable[[ToastDismissedEventArgs], None]] = ...,
        on_failed: Optional[Callable[[ToastFailedEventArgs], None]] = ...,
        actions: Iterable[Union[ToastButton, ToastSystemButton]] = ...,
        images: Iterable[ToastDisplayImage] = ...,
        inputs: Iterable[ToastInput] = ...,
    ) -> None:
        """
        Initialise a toast

        :param actions: Iterable of actions to add; see :meth:`AddAction`
        :type actions: Iterable[Union[ToastButton, ToastSystemButton]]
        :param images: See :meth:`AddImage`
        :type images: Iterable[ToastDisplayImage]
        :param inputs: See :meth:`AddInput`
        :type inputs: Iterable[ToastInput]
        """
        ...

    def __eq__(self, other) -> bool: ...
    def __repr__(self):  # -> str:
        ...
    def AddAction(self, action: Union[ToastButton, ToastSystemButton]) -> None:
        """
        Add an action to the action list. For example, if you're setting up a reminder,
        you would use 'action=remindlater&date=2020-01-20' as arguments. Maximum of five.

        :type action: Union[ToastButton, ToastSystemButton]
        """
        ...

    def AddImage(self, image: ToastDisplayImage) -> None:
        """
        Adds an the image that will be displayed on the toast.
        If using WindowsToaster, a maximum of two (one as the logo and one hero) images will work.

        :param image: :class:`ToastDisplayImage` to display in the toast
        """
        ...

    def AddInput(self, toast_input: ToastInput) -> None:
        """
        Adds an input field to the notification. It will be supplied as user_input of type ValueSet in on_activated

        :param toast_input: :class:`ToastInput` to display in the toast
        """
        ...

    @property
    def launch_action(self) -> Optional[str]:
        """Protocol to launch when the toast is clicked"""
        ...

    @launch_action.setter
    def launch_action(self, value: Optional[str]):  # -> None:
        ...
    def clone(self) -> Toast:
        """
        Clone the current toast and return the new one

        :return: A deep copy of the toast
        :rtype: Toast
        """
        ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common.util.text import escape_iri_path_segment
from gajim.common.util.text import format_bytes_as_hex
from gajim.common.util.text import format_duration
from gajim.common.util.text import jid_to_iri
from gajim.common.util.text import remove_invalid_xml_chars


class Test(unittest.TestCase):
    def test_remove_invalid_xml_chars(self) -> None:
        invalid_chars = ["\x0b", "\udfff", "\x08"]
        for char in invalid_chars:
            self.assertEqual(remove_invalid_xml_chars(char), "")

        self.assertEqual(remove_invalid_xml_chars(""), "")
        self.assertEqual(remove_invalid_xml_chars("ä"), "ä")

    def test_escape_iri_path_segment(self) -> None:
        self.assertEqual(escape_iri_path_segment(""), "", "<empty string>")

        ueber = "u\u0308ber"
        self.assertEqual(escape_iri_path_segment(ueber), ueber)

        segment = "".join(chr(c) for c in range(0x20, 0x7F))
        self.assertEqual(
            escape_iri_path_segment(segment),
            "%20!%22%23$%25&'()*+,-.%2F0123456789:;%3C=%3E%3F@ABCDEFGHIJKLMN"
            "OPQRSTUVWXYZ%5B%5C%5D%5E_%60abcdefghijklmnopqrstuvwxyz%7B%7C%7D~",
            "ASCII printable",
        )

        self.assertEqual(
            escape_iri_path_segment(
                "".join(chr(c) for c in range(0x01, 0x20)) + chr(0x7F)
            ),
            "".join("%%%02X" % c for c in range(0x01, 0x20)) + "%7F",
            "ASCII control (no null)",
        )

    def test_jid_to_iri(self) -> None:
        jid = r"foo@bar"
        self.assertEqual(jid_to_iri(jid), rf"xmpp:{jid}", jid)
        jid = r"my\20self@[::1]/home"
        self.assertEqual(jid_to_iri(jid), r"xmpp:my%5C20self@%5B::1%5D/home", jid)

    def test_format_duration_width(self) -> None:
        def do(total_seconds: float, expected: str) -> None:
            self.assertEqual(format_duration(0.0, total_seconds * 1e9), expected)

        do(0, "0:00")
        do(60, "0:00")
        do(10 * 60, "00:00")
        do(60 * 60, "0:00:00")
        do(10 * 60 * 60, "00:00:00")
        do(100 * 60 * 60, "000:00:00")

    def test_format_duration(self) -> None:
        def do(duration: float, expected: str) -> None:
            self.assertEqual(format_duration(duration, 100 * 60 * 60 * 1e9), expected)

        do(1.0, "000:00:00")
        do(999999999.0, "000:00:00")
        do(1000000000.0, "000:00:01")
        do(59999999999.0, "000:00:59")
        do(60000000000.0, "000:01:00")
        do(3599999999999.0, "000:59:59")
        do(3600000000000.0, "001:00:00")
        do(3599999999999999.0, "999:59:59")

    def test_format_bytes_as_hex(self) -> None:
        tests = [
            (
                b"U\xe9\xf4\xd6~\x17\xa6\xdf]\xc7\xfe\x9a\x01\r\x8b\xea\xfd\x8f@.",
                "55:E9:F4:D6:7E:17:A6:DF:5D:C7\nFE:9A:01:0D:8B:EA:FD:8F:40:2E",
            ),
        ]

        for test in tests:
            bytes_, string = test
            hex_string = format_bytes_as_hex(bytes_, 2)
            self.assertEqual(hex_string, string)


if __name__ == "__main__":
    unittest.main()



"""
This type stub file was generated by pyright.
"""

import typing

from ._util.gtk import GtkIcon

class Icon(GtkIcon):
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...



#!/usr/bin/env python3

from __future__ import annotations

import argparse
import gzip
import logging
import shutil
import subprocess
from pathlib import Path

logging.basicConfig(level="INFO", format="%(message)s")
log = logging.getLogger(__name__)

CONFIG_VALUES = {
    "APP_ID": "org.gajim.Gajim",
    "APP_NAME": "Gajim",
    "APP_ICON_NAME": "org.gajim.Gajim",
    "FLATPAK": "False",
    "FLATPAK_NIGHTLY": "False",
}

CONFIG = Path("gajim/config.py.in")
METAINFO = Path("data/org.gajim.Gajim.metainfo.xml.in")
DESKTOP = Path("data/org.gajim.Gajim.desktop.in")
METADATA = Path("dist/metadata")

MAN_FILES = [Path("data/gajim.1"), Path("data/gajim-remote.1")]

META_FILES = [
    (Path("data/org.gajim.Gajim.desktop"), "--desktop"),
    (Path("data/org.gajim.Gajim.metainfo.xml"), "--xml"),
]

ICONS = [
    Path("gajim/data/icons/hicolor/scalable/apps/gajim.svg"),
    Path("gajim/data/icons/hicolor/scalable/apps/gajim-symbolic.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-away.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-chat.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-connecting.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-dnd.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-message-new.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-offline.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-online.svg"),
    Path("gajim/data/icons/hicolor/scalable/status/gajim-status-xa.svg"),
]

INSTALL_FILES = {
    "gajim-remote.1.gz": "share/man/man1",
    "gajim.1.gz": "share/man/man1",
    "org.gajim.Gajim.desktop": "share/applications",
    "org.gajim.Gajim.metainfo.xml": "share/metainfo",
    "org.gajim.Gajim-symbolic.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim.svg": "share/icons/hicolor/scalable/apps",
}

INSTALL_FLATPAK_ICONS = {
    "org.gajim.Gajim-status-away.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-chat.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-connecting.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-dnd.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-message-new.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-offline.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-online.svg": "share/icons/hicolor/scalable/apps",
    "org.gajim.Gajim-status-xa.svg": "share/icons/hicolor/scalable/apps",
}


def configure_file(path: Path, values: dict[str, str]) -> None:
    log.info("Configure %s", path)
    content = path.read_text()
    for old, new in values.items():
        content = content.replace(f"@{old}@", new)
    path = path.with_suffix("")
    path.write_text(content)


def configure_flatpak() -> None:
    CONFIG_VALUES["FLATPAK"] = "True"


def configure_flatpak_nightly() -> None:
    CONFIG_VALUES["APP_ID"] = "org.gajim.Gajim.Devel"
    CONFIG_VALUES["APP_NAME"] = "Gajim (Nightly)"
    CONFIG_VALUES["APP_ICON_NAME"] = "org.gajim.Gajim.Devel"
    CONFIG_VALUES["FLATPAK"] = "True"
    CONFIG_VALUES["FLATPAK_NIGHTLY"] = "True"


def build_man() -> None:
    # Build man files in target path

    for man_path in MAN_FILES:
        data = man_path.read_bytes()
        man_file_name = man_path.name

        man_out_path = METADATA / f"{man_file_name}.gz"
        log.info("Compress %s >> %s", man_file_name, man_out_path)

        with gzip.GzipFile(man_out_path, "wb", mtime=0) as f_out:
            f_out.write(data)


def build_meta() -> None:
    for file_path, option in META_FILES:
        out_path = METADATA / file_path.name

        log.info("Compile %s >> %s", file_path, out_path)

        subprocess.run(
            [
                "msgfmt",
                option,
                "-d",
                "po",
                "--template",
                str(file_path),
                "-o",
                str(out_path),
            ],
            check=True,
        )


def build_app_icons() -> None:
    for file_path in ICONS:
        icon_name = file_path.name.replace("gajim", "org.gajim.Gajim")
        out_path = METADATA / icon_name

        log.info("Copy %s >> %s", file_path, out_path)
        shutil.copy2(file_path, out_path)


def build_translations() -> None:
    # Compile translation files and place them into "gajim/data/locale"

    source_dir = Path.cwd()
    translation_dir = source_dir / "po"
    locale_dir = source_dir / "gajim" / "data" / "locale"

    langs = sorted([lang.stem for lang in translation_dir.glob("*.po")])

    for lang in langs:
        po_file = source_dir / "po" / f"{lang}.po"
        mo_file = locale_dir / lang / "LC_MESSAGES" / "gajim.mo"
        mo_file.parent.mkdir(parents=True, exist_ok=True)

        log.info("Compile %s >> %s", po_file, mo_file)

        subprocess.run(["msgfmt", str(po_file), "-o", str(mo_file)], check=True)


def install(*, prefix: Path, files: dict[str, str]) -> None:
    for file, path in files.items():
        src = METADATA / file
        dest_dir = prefix / path
        log.info("Copy %s to %s", src, dest_dir)
        if not dest_dir.exists():
            dest_dir.mkdir(parents=True)
        shutil.copy(src, dest_dir / file)


def rename(*, prefix: Path, files: dict[str, str], new_app_id: str) -> None:
    for file_name, path in files.items():
        dest_dir = prefix / path
        file = dest_dir / file_name
        new_file_name = file_name.replace("org.gajim.Gajim", new_app_id)
        new_file = dest_dir / new_file_name
        log.info("Rename File %s to %s", file, new_file)
        file.rename(new_file)


def cmd_build(args: argparse.Namespace) -> None:
    if args.dist in ["win", "macos"]:
        build_translations()
        return

    if args.dist == "flatpak":
        configure_flatpak()
    elif args.dist == "flatpak-nightly":
        configure_flatpak_nightly()

    configure_file(CONFIG, CONFIG_VALUES)
    configure_file(METAINFO, CONFIG_VALUES)
    configure_file(DESKTOP, CONFIG_VALUES)

    METADATA.mkdir(parents=True, exist_ok=True)

    build_translations()
    build_man()
    build_meta()
    build_app_icons()


def cmd_install(args: argparse.Namespace) -> None:
    install(prefix=args.prefix, files=INSTALL_FILES)

    if args.dist.startswith("flatpak"):
        install(prefix=args.prefix, files=INSTALL_FLATPAK_ICONS)

    if args.dist == "flatpak-nightly":
        rename(
            prefix=args.prefix, files=INSTALL_FILES, new_app_id="org.gajim.Gajim.Devel"
        )
        rename(
            prefix=args.prefix,
            files=INSTALL_FLATPAK_ICONS,
            new_app_id="org.gajim.Gajim.Devel",
        )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(
        required=True, metavar="commands", dest="command"
    )

    build_parser = subparsers.add_parser("build", help="Build Gajim")
    build_parser.add_argument(
        "--dist",
        choices=["unix", "flatpak", "flatpak-nightly", "win", "macos"],
        default="unix",
        help="Distribution",
    )
    build_parser.set_defaults(func=cmd_build)

    install_parser = subparsers.add_parser("install", help="Install metadata files")
    install_parser.add_argument(
        "--dist",
        choices=["unix", "flatpak", "flatpak-nightly"],
        default="unix",
        help="Distribution",
    )
    install_parser.add_argument(
        "--prefix",
        type=Path,
        default="/usr",
        help='The path prefix for installation (e.g. "/usr")',
    )
    install_parser.set_defaults(func=cmd_install)

    args = parser.parse_args()
    args.func(args)



#!/usr/bin/env python3

import argparse
import os
import subprocess
import sys
from pathlib import Path

REPO_DIR = Path(__file__).resolve().parent.parent
ICONS_DIR = REPO_DIR / "gajim" / "data" / "icons" / "hicolor" / "scalable" / "devices"


def convert_svg(input_path: Path, temp_path: Path) -> None:
    # https://wiki.inkscape.org/wiki/Action
    try:
        subprocess.check_output(
            [
                "inkscape",
                input_path,
                "--actions",
                "select-all;object-stroke-to-path",
                "--export-type=svg",
                "--export-plain-svg",
                f"--export-filename={temp_path}",
            ],
        )
    except Exception as e:
        print("Error:", e)
        sys.exit(1)


def clean_svg(temp_path: Path, output_path: Path) -> None:
    try:
        subprocess.check_output(
            [
                "scour",
                temp_path,
                output_path,
                "--strip-xml-prolog",
                "--no-line-breaks",
                "--enable-id-stripping",
            ],
        )
    except Exception as e:
        print("Error:", e)
        sys.exit(1)


def process_file(file_name: str) -> None:
    print("Processing", file_name)
    input_path = ICONS_DIR / file_name
    temp_path = ICONS_DIR / f"{file_name}-temp.svg"
    output_path = ICONS_DIR / f"lucide-{Path(file_name).stem}-symbolic.svg"

    convert_svg(input_path, temp_path)
    clean_svg(temp_path, output_path)

    # Remove temp file and original file
    temp_path.unlink()
    input_path.unlink()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="command")

    file_ = subparsers.add_parser("file", help="Process single file")
    file_.add_argument("file_name")

    all_files = subparsers.add_parser("all_files")
    args = parser.parse_args()

    if args.command == "all_files":
        for _root, _dirs, files in os.walk(ICONS_DIR):
            for file in sorted(files):
                file_path = Path(file)

                if not file_path.suffix == ".svg":
                    continue

                if file_path.stem.startswith("lucide") or file_path.stem.endswith(
                    "symbolic"
                ):
                    continue

                process_file(file)

    elif args.command == "file":
        process_file(args.file_name)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Encryption
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.storage import MessageArchiveStorage


class EncryptionTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account_jid = JID.from_string("user@domain.org")
        self._account = "testacc1"
        self._remote_jid = JID.from_string("remote@jid.org")
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_encryption_join(self):
        enc_data1 = Encryption(protocol="OMEMO", key="testkey", trust=2)

        enc_data2 = Encryption(protocol="OMEMO", key="testkey", trust=2)

        message_data1 = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="1",
            stanza_id=get_uuid(),
            encryption_=enc_data1,
        )

        message_data2 = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(1, UTC),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some other Message",
            id="2",
            stanza_id=get_uuid(),
            encryption_=enc_data2,
        )

        message_pk1 = self._archive.insert_object(message_data1)
        message_pk2 = self._archive.insert_object(message_data2)

        message1 = self._archive.get_message_with_pk(message_pk1)
        message2 = self._archive.get_message_with_pk(message_pk2)

        assert message1 is not None
        assert message1.encryption is not None

        assert message2 is not None
        assert message2.encryption is not None

        self.assertEqual(message1.encryption.pk, message2.encryption.pk)
        self.assertEqual(message1.encryption.protocol, "OMEMO")
        self.assertEqual(message1.encryption.key, "testkey")
        self.assertEqual(message1.encryption.trust, 2)

    def test_encryption_update(self):
        enc_data1 = Encryption(protocol="OMEMO", key="testkey1", trust=2)

        enc_data2 = Encryption(protocol="OMEMO", key="testkey1", trust=2)

        pk1 = self._archive.insert_row(enc_data1, return_pk_on_conflict=True)
        pk2 = self._archive.insert_row(enc_data2, return_pk_on_conflict=True)

        self.assertEqual(pk1, pk2)

        with self._archive.get_session() as s:
            res = s.scalar(select(Encryption).where(Encryption.pk == pk1))
        assert res is not None


if __name__ == "__main__":
    unittest.main()



"""
This type stub file was generated by pyright.
"""

import ctypes

windll = ...
WM_CREATE = ...
WM_DISPLAYCHANGE = ...
WM_NCCREATE = ...
WM_LBUTTONUP = ...
WM_RBUTTONUP = ...
WM_USER = ...
WM_STOP = ...
WM_NOTIFY = ...
LR_DEFAULTSIZE = ...
LR_LOADFROMFILE = ...
MFS_CHECKED = ...
MFS_DEFAULT = ...
MFS_DISABLED = ...
MFS_ENABLED = ...
MFS_GRAYED = ...
MFS_HILITE = ...
MFS_UNCHECKED = ...
MFS_UNHILITE = ...
MFT_BITMAP = ...
MFT_MENUBARBREAK = ...
MFT_MENUBREAK = ...
MFT_OWNERDRAW = ...
MFT_RADIOCHECK = ...
MFT_RIGHTJUSTIFY = ...
MFT_RIGHTORDER = ...
MFT_SEPARATOR = ...
MFT_STRING = ...
MIIM_BITMAP = ...
MIIM_CHECKMARKS = ...
MIIM_DATA = ...
MIIM_FTYPE = ...
MIIM_ID = ...
MIIM_STATE = ...
MIIM_STRING = ...
MIIM_SUBMENU = ...
MIIM_TYPE = ...
MSGFLT_ALLOW = ...
MSGFLT_DISALLOW = ...
MSGFLT_RESET = ...
NIF_MESSAGE = ...
NIF_ICON = ...
NIF_TIP = ...
NIF_STATE = ...
NIF_INFO = ...
NIF_GUID = ...
NIF_REALTIME = ...
NIF_SHOWTIP = ...
NIM_ADD = ...
NIM_MODIFY = ...
NIM_DELETE = ...
NIM_SETFOCUS = ...
NIM_SETVERSION = ...
TPM_CENTERALIGN = ...
TPM_LEFTALIGN = ...
TPM_RIGHTALIGN = ...
TPM_BOTTOMALIGN = ...
TPM_TOPALIGN = ...
TPM_VCENTERALIGN = ...
TPM_NONOTIFY = ...
TPM_RETURNCMD = ...
TPM_LEFTBUTTON = ...
TPM_RIGHTBUTTON = ...
TPM_HORNEGANIMATION = ...
TPM_HORPOSANIMATION = ...
TPM_NOANIMATION = ...
TPM_VERNEGANIMATION = ...
TPM_VERPOSANIMATION = ...
TPM_HORIZONTAL = ...
TPM_VERTICAL = ...
WS_POPUP = ...
PM_NOREMOVE = ...
COLOR_WINDOW = ...
HWND_MESSAGE = ...
IMAGE_ICON = ...
LPMSG = ...
LPPOINT = ...
WNDPROC = ...

class MENUITEMINFO(ctypes.Structure):
    _fields_ = ...

LPMENUITEMINFO = ...

class NOTIFYICONDATAW(ctypes.Structure):
    class VERSION_OR_TIMEOUT(ctypes.Union):
        _fields_ = ...

    class GUID(ctypes.Structure):
        _fields_ = ...

    _fields_ = ...
    _anonymous_ = ...

LPNOTIFYICONDATAW = ...

class TPMPARAMS(ctypes.Structure):
    _fields_ = ...

LPTPMPARAMS = ...

class WNDCLASSEX(ctypes.Structure):
    _fields_ = ...

LPWNDCLASSEX = ...
CreatePopupMenu = ...
CreateWindowEx = ...
DefWindowProc = ...
DestroyIcon = ...
DestroyMenu = ...
DestroyWindow = ...
DispatchMessage = ...
GetCursorPos = ...
GetMessage = ...
GetModuleHandle = ...
InsertMenuItem = ...
LoadImage = ...
PeekMessage = ...
PostMessage = ...
PostQuitMessage = ...
RegisterClassEx = ...
SetForegroundWindow = ...
Shell_NotifyIcon = ...
TranslateMessage = ...
TrackPopupMenuEx = ...
UnregisterClass = ...
RegisterWindowMessage = ...
WM_TASKBARCREATED = ...
ChangeWindowMessageFilterEx = ...



#!/usr/bin/env python3

import argparse
import re
import subprocess
import sys
from datetime import datetime
from pathlib import Path

REPO_DIR = Path(__file__).resolve().parent.parent


INIT = REPO_DIR / "gajim" / "__init__.py"
FLATPAK = REPO_DIR / "flatpak" / "org.gajim.Gajim.yaml"
APPDATA = REPO_DIR / "data" / "org.gajim.Gajim.metainfo.xml.in"
CHANGELOG = REPO_DIR / "ChangeLog"

VERSION_RX = r"\d+\.\d+\.\d+"


def get_current_version() -> str:
    with INIT.open("r") as f:
        content = f.read()

    match = re.search(VERSION_RX, content)
    if match is None:
        sys.exit("Unable to find current version")
    return match[0]


def bump_init(current_version: str, new_version: str) -> None:
    with INIT.open("r", encoding="utf8") as f:
        content = f.read()

    content = content.replace(current_version, new_version, 1)

    with INIT.open("w", encoding="utf8") as f:
        f.write(content)


def bump_flatpak(current_version: str, new_version: str) -> None:
    with FLATPAK.open("r", encoding="utf8") as f:
        content = f.read()

    content = content.replace(f"tag: {current_version}", f"tag: {new_version}", 1)

    with FLATPAK.open("w", encoding="utf8") as f:
        f.write(content)


def bump_appdata(new_version: str) -> None:
    with APPDATA.open("r", encoding="utf8") as f:
        lines = f.readlines()

    date = datetime.today().strftime("%Y-%m-%d")
    release_url = f"https://dev.gajim.org/gajim/gajim/-/blob/{new_version}/ChangeLog"
    release_string = (
        f'    <release version="{new_version}" date="{date}">\n'
        f'      <url type="details">{release_url}</url>\n'
        "    </release>"
    )

    with APPDATA.open("w", encoding="utf8") as f:
        for line in lines:
            f.write(line)
            if "<releases>" in line:
                f.write(release_string)
                f.write("\n")


def make_changelog(new_version: str) -> None:
    cmd = ["git-chglog", "--next-tag", new_version]

    result = subprocess.run(
        cmd, cwd=REPO_DIR, text=True, check=True, capture_output=True
    )

    changes = result.stdout
    changes = changes.removeprefix("\n")

    current_changelog = CHANGELOG.read_text()

    with CHANGELOG.open("w") as f:
        f.write(changes + current_changelog)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Bump Version")
    parser.add_argument("version", help="The new version, e.g. 1.5.0")
    args = parser.parse_args()

    current_version = get_current_version()
    bump_init(current_version, args.version)
    bump_flatpak(current_version, args.version)
    bump_appdata(args.version)
    make_changelog(args.version)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from unittest.mock import MagicMock

from gi.repository import Gtk

from gajim.common import app

from gajim.gtk.chat_filter import ChatFilter
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT1 = "account1"
ACCOUNT2 = "account2"


class TestRosterModule:
    def __init__(self, account: str) -> None:
        self._account = account

    def get_groups(self) -> set[str]:
        if self._account == ACCOUNT1:
            return {"Group A", "Group B"}
        if self._account == ACCOUNT2:
            return {"Group B", "Group C"}
        return set()


class TestClient:
    def __init__(self, account: str) -> None:
        self._account = account

        self._roster_module = TestRosterModule(self._account)

    def get_module(self, module: str) -> Any:
        if module == "Roster":
            return self._roster_module
        return MagicMock()


class TestChatFilter(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(box)

        self._chat_filter = ChatFilter()
        self._chat_filter.connect("filter-changed", self._on_filter_changed)
        box.append(self._chat_filter)

        reset_button = Gtk.Button(label="Reset")
        reset_button.connect("clicked", self._on_reset_clicked)
        box.append(reset_button)

    def _on_filter_changed(self, chat_filter: ChatFilter) -> None:
        print("Filters:", chat_filter.get_filters())

    def _on_reset_clicked(self, _button: Gtk.Button) -> None:
        self._chat_filter.reset()

    def _cleanup(self) -> None:
        pass


util.init_settings()

app.get_client = MagicMock(side_effect=TestClient)
app.get_connected_accounts = MagicMock(return_value=[ACCOUNT1, ACCOUNT2])

window = TestChatFilter()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.modules.contacts import BareContact
from gajim.common.storage.cache import CacheStorage


class ContactTest(unittest.TestCase):
    def test_contact_object(self):
        account = "testacc"
        jid = JID.from_string("user@example.org")

        app.storage.cache = CacheStorage(in_memory=True)
        app.storage.cache.init()

        contact = BareContact(MagicMock(), jid, account)
        contact2 = BareContact(MagicMock(), jid, account)

        self.assertEqual(contact, contact2)
        self.assertEqual(hash(contact), hash(contact2))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3


import json
import logging
import sys
from urllib.request import urlopen

import ruamel.yaml

PYPI_INDEX = "https://pypi.org/pypi"

YAML = ruamel.yaml.YAML()
YAML.indent(mapping=2, sequence=4, offset=2)
YAML.preserve_quotes = True
YAML.width = 180

logging.basicConfig(level="INFO", format="%(levelname)s: %(message)s")
log = logging.getLogger(__name__)


def get_package_type(module: ruamel.yaml.comments.CommentedMap) -> str:
    url = module["sources"][0]["url"]
    if ".whl" in url:
        return "bdist_wheel"
    return "sdist"


def get_current_version(
    module: ruamel.yaml.comments.CommentedMap,
) -> tuple[str | None, str | None]:
    name = module["name"].replace("python3-", "")
    url = module["sources"][0]["url"]
    if url.endswith(".git"):
        return None, None
    file_name = url.split("/")[-1]
    file_name = file_name.replace(name, "")
    parts = file_name.split("-")
    pkg_version = parts[1].replace(".tar.gz", "")
    py_version = parts[2] if ".tar.gz" not in parts[1] else None
    return pkg_version, py_version


def get_latest_version(
    package_name: str, package_type: str, py_version: str | None
) -> tuple[str, str, str, str]:
    with urlopen(f"{PYPI_INDEX}/{package_name}/json") as f:
        data = f.read()
        d = json.loads(data)
        version = d["info"]["version"]
        sha = None
        for entry in d["releases"][version]:
            if entry["packagetype"] != package_type:
                continue

            if py_version is not None and entry["python_version"] != py_version:
                continue

            if sha is not None:
                raise ValueError("No sha found")

            sha = entry["digests"]["sha256"]
            url = entry["url"]
            filename = entry["filename"]
            return version, sha, url, filename

    raise ValueError("Package not found")


def update_module(module: ruamel.yaml.comments.CommentedMap) -> None:
    if not module["name"].startswith("python3-"):
        log.warning("Check %s manually", module["name"])
        return

    if "only-arches" in module:
        log.warning("Update %s manually", module["name"])
        return

    package_type = get_package_type(module)
    current_version, py_version = get_current_version(module)
    if current_version is None:
        print("Current version None")
        return

    name = module["name"].replace("python3-", "")
    _latest_version, sha, url, filename = get_latest_version(
        name, package_type, py_version
    )

    log.info("Update %s", name)
    "".rsplit(" ", maxsplit=1)  # noqa: SIM905
    if module["sources"][0]["type"] == "file":
        command = module["build-commands"][0]
        current_filename = command.rsplit(" ", maxsplit=1)[1]
        module["build-commands"][0] = command.replace(current_filename, filename)

    module["sources"][0]["url"] = url
    module["sources"][0]["sha256"] = sha


def update_modules(data) -> None:
    for module in data["modules"]:
        if isinstance(module, ruamel.yaml.comments.CommentedMap):
            update_module(module)


if __name__ == "__main__":
    if len(sys.argv) != 2:
        sys.exit("Input yaml file is missing")

    try:
        with open(sys.argv[1]) as f:
            data = YAML.load(f)
    except (
        FileNotFoundError,
        ruamel.yaml.parser.ParserError,
        ruamel.yaml.scanner.ScannerError,
    ) as error:
        sys.exit(error)

    update_modules(data)

    with open(sys.argv[1], "w") as f:
        YAML.dump(data, f)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import tempfile

from gi.repository import Gtk

from gajim.common import configpaths

from gajim.gtk.manage_sounds import ManageSounds

from . import util

ACCOUNT = "test"

util.init_settings()

configpaths.set_separation(True)
configpaths.set_config_root(tempfile.gettempdir())
configpaths.init()

window = ManageSounds(Gtk.Window())
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-only

import signal
import sys
from types import FrameType

from gi.repository import Adw
from nbxmpp.protocol import JID

import gajim.common.i18n
from gajim.common import app
from gajim.common import configpaths
from gajim.common.application import CoreApplication

from gajim.gtk.avatar import AvatarStorage
from gajim.gtk.util.icons import get_icon_theme


class GajimTestApplication(Adw.Application, CoreApplication):
    def __init__(self) -> None:
        CoreApplication.__init__(self)
        Adw.Application.__init__(self, application_id="org.gajim.GajimTest")

        self.avatar_storage = AvatarStorage()

        self.connect("startup", self._on_startup)

    def _init_signals(self) -> None:
        def sigint_cb(num: int, stack: FrameType | None) -> None:
            print(" SIGINT/SIGTERM received")
            self.start_shutdown()

        # ^C exits the application normally
        signal.signal(signal.SIGINT, sigint_cb)
        signal.signal(signal.SIGTERM, sigint_cb)
        if sys.platform != "win32":
            signal.signal(signal.SIGPIPE, signal.SIG_DFL)

    def _on_startup(self, _application: Adw.Application) -> None:
        gajim.common.i18n.init()
        self._init_signals()

        configpaths.set_config_root(str(configpaths.get_temp_dir()))
        if not self._init_core(in_memory=True):
            return

        app.load_css_config()

        icon_theme = get_icon_theme()
        icon_theme.add_search_path(str(configpaths.get("ICONS")))

    def add_account(self, name: str) -> None:
        self.create_account(
            name, JID.from_string(f"{name}@example.com"), "", None, None
        )
        self.enable_account(name, connect=False)

        app.css_config.refresh()

    def start_shutdown(self) -> None:
        self._start_shutdown()

    def _shutdown_complete(self) -> None:
        CoreApplication._shutdown_complete(self)
        self.quit()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import cast

import time

from gi.repository import GLib
from gi.repository import Gtk

from gajim.common import app
from gajim.common.events import DBMigrationError
from gajim.common.events import DBMigrationFinished
from gajim.common.events import DBMigrationProgress
from gajim.common.events import DBMigrationStart

from gajim.gtk.db_migration import DBMigration

from . import util


class DBMigrationTest:
    def __init__(self) -> None:
        self._progressing = False

        window = DBMigration()
        box = cast(Gtk.Box, window.get_content())
        box.set_orientation(Gtk.Orientation.VERTICAL)

        button_box = Gtk.Box(spacing=12, halign=Gtk.Align.CENTER)
        box.append(button_box)

        progress_button = Gtk.Button(label="Progress")
        progress_button.connect("clicked", self._on_progress_clicked)
        button_box.append(progress_button)

        finish_button = Gtk.Button(label="Finish")
        finish_button.connect("clicked", self._on_finish_clicked)
        button_box.append(finish_button)

        error_button = Gtk.Button(label="Error")
        error_button.connect("clicked", self._on_error_clicked)
        button_box.append(error_button)

        window.show()

        GLib.timeout_add_seconds(3, self._add_window)

        self._progress()

    def _add_window(self) -> None:
        # Simulate main window becoming available
        app.window = Gtk.Window(width_request=800, height_request=800)
        app.window.show()

    def _progress(self) -> None:
        app.ged.raise_event(DBMigrationStart(version=1))
        self._progressing = True

        count = 100000
        for i in range(100000):
            if not self._progressing:
                break

            time.sleep(0.0000001)
            if i % 1000 == 0:
                app.ged.raise_event(DBMigrationProgress(count=count, progress=i))

    def _on_progress_clicked(self, _button: Gtk.Button) -> None:
        self._progress()

    def _on_finish_clicked(self, _button: Gtk.Button) -> None:
        self._progressing = False

        app.ged.raise_event(DBMigrationFinished())

    def _on_error_clicked(self, _button: Gtk.Button) -> None:
        self._progressing = False

        try:
            test = 1 / 0  # type: ignore # noqa: F841
        except ZeroDivisionError as e:
            app.ged.raise_event(DBMigrationError(exception=e))


util.init_settings()

DBMigrationTest()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from types import TracebackType
from unittest.mock import MagicMock

from gajim.common import app

from gajim.gtk.exception import ExceptionDialog

from . import util


def _create_traceback(message: str) -> TracebackType | None:
    tb = None
    depth = 0
    while True:
        try:
            frame = sys._getframe(depth)  # type: ignore
            depth += 1
        except ValueError as _exc:
            break

        tb = TracebackType(tb, frame, frame.f_lasti, frame.f_lineno)

    return tb


tb = _create_traceback("Test")
assert isinstance(tb, TracebackType)

util.init_settings()

app.is_installed = MagicMock(return_value=True)

window = ExceptionDialog(BaseException, Exception(), tb)
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID
from nbxmpp.structs import VoiceRequest

from gajim.common import app
from gajim.common.modules.contacts import GroupchatContact

from gajim.gtk.groupchat_voice_requests_button import VoiceRequestsButton
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
FROM_JID = "groupchat@conference.example.org"


class TestMUCModule:
    def __init__(self, account: str) -> None:
        self._account = account

        self._requests: list[VoiceRequest] = []
        for item in range(15):
            self._requests.append(
                VoiceRequest(
                    JID.from_string(f"user{item}@conference.example.org/user"),
                    f"Nick #{item}",
                    None,
                )
            )
        self._requests.append(
            VoiceRequest(
                JID.from_string(
                    "extralongjid_with_many_characters@conference.example.org/with_resource"
                ),
                "Nickname with many characters very very looooooooong",
                None,
            )
        )

    def get_voice_requests(self, _contact: GroupchatContact) -> list[VoiceRequest]:
        return self._requests

    def approve_voice_request(
        self, _contact: GroupchatContact, request: VoiceRequest
    ) -> None:
        self._requests.remove(request)

    def decline_voice_request(
        self, _contact: GroupchatContact, request: VoiceRequest
    ) -> None:
        self._requests.remove(request)


class TestClient:
    def __init__(self, account: str) -> None:
        self._account = account

        self._muc_module = TestMUCModule(self._account)

    def get_module(self, module: str) -> Any:
        if module == "MUC":
            return self._muc_module
        return MagicMock()


class TestGroupChatVoiceRequestButton(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        self._box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(self._box)

        button = VoiceRequestsButton()
        self._box.append(button)

        contact = self._get_contact()
        button.switch_contact(contact)

    def _get_contact(self) -> GroupchatContact:
        contact = MagicMock(spec_set=GroupchatContact)
        contact.account = ACCOUNT
        contact.jid = JID.from_string(FROM_JID)
        contact.name = "Test Contact"
        contact.is_groupchat = True
        return contact

    def _cleanup(self) -> None:
        pass


test_client = TestClient(ACCOUNT)
app.get_client = MagicMock(return_value=test_client)

util.init_settings()

window = TestGroupChatVoiceRequestButton()
window.show()

util.run_app()



class KeyringError(Exception): ...
class PasswordSetError(KeyringError): ...
class PasswordDeleteError(KeyringError): ...
class InitError(KeyringError): ...
class KeyringLocked(KeyringError): ...
class NoKeyringError(KeyringError): ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import GroupchatContact

from gajim.gtk.chat_function_page import ChatFunctionPage
from gajim.gtk.chat_function_page import FunctionMode
from gajim.gtk.dropdown import GajimDropDown
from gajim.gtk.dropdown import KeyValueItem
from gajim.gtk.window import GajimAppWindow

from . import util

FUNCTION_DATA = {
    FunctionMode.KICK: "Nasty User",
    FunctionMode.BAN: "Nasty User",
    FunctionMode.CAPTCHA_ERROR: "Captcha error message",
    FunctionMode.JOIN_FAILED: "registration-required",
    FunctionMode.CREATION_FAILED: "Creation failed message",
    FunctionMode.CONFIG_FAILED: "Config failed message",
}

ACCOUNT = "testacc1"
FROM_JID = "test@example.org"


class TestChatFunctionPage(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=700,
            default_height=700,
            add_window_padding=True,
            header_bar=True,
        )

        self._main_box = Gtk.Box(
            hexpand=True,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            orientation=Gtk.Orientation.VERTICAL,
        )

        self._chat_function_page = ChatFunctionPage()
        self._chat_function_page.connect("finish", self._on_finish)
        self._chat_function_page.connect("message", self._on_message)

        self._main_box.append(self._chat_function_page)

        drop_down: GajimDropDown[str] = GajimDropDown(
            [item.name for item in FunctionMode]
        )
        drop_down.connect("notify::selected", self._on_function_mode_selected)
        drop_down.set_selected(1)
        self._main_box.append(drop_down)

        self.set_child(self._main_box)

    def _on_function_mode_selected(
        self, drop_down: GajimDropDown[str], *args: Any
    ) -> None:
        selected_mode = drop_down.get_selected_item()
        assert isinstance(selected_mode, KeyValueItem)
        mode = selected_mode.key
        data = FUNCTION_DATA.get(FunctionMode[selected_mode.key])
        files = ["test.txt"]

        contact_type = "bare"
        if FunctionMode[mode] == FunctionMode.CHANGE_NICKNAME:
            contact_type = "groupchat"

        contact = self._get_contact(contact_type)

        self._chat_function_page.set_mode(contact, FunctionMode[mode], data, files)

    def _get_contact(self, contact_type: str) -> BareContact | GroupchatContact:
        if contact_type == "bare":
            contact = MagicMock(spec_set=BareContact)
        else:
            contact = MagicMock(spec_set=GroupchatContact)
            contact.nickname = "Nickname"

        contact.account = ACCOUNT
        contact.jid = JID.from_string(FROM_JID)
        contact.name = "Example Contact"
        return contact

    def _on_finish(self, widget: ChatFunctionPage, finished: bool) -> None:
        print("Close control", finished)

    def _on_message(self, widget: ChatFunctionPage, message: str) -> None:
        print("Message", message)

    def _cleanup(self) -> None:
        pass


util.init_settings()

app.get_client = MagicMock()
app.account_is_connected = MagicMock(return_value=True)

app.window = MagicMock()
app.window.get_preferred_ft_method = MagicMock(return_value="httpupload")

window = TestChatFunctionPage()
window.show()

util.run_app()



#!/usr/bin/env python3

import argparse
import csv
import io
import logging
import re
import sys
from datetime import datetime
from datetime import UTC
from pathlib import Path
from urllib.request import urlopen

SCHEMES_URL = "https://www.iana.org/assignments/uri-schemes/uri-schemes-1.csv"

BOILER_PLATE = """# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-only

# Generated by get_iana_data.py @ {current_date}

"""

logging.basicConfig(level="INFO", format="%(levelname)s: %(message)s")
log = logging.getLogger(__name__)


def download_file(url: str) -> str:
    log.info("Download: %s", url)
    with urlopen(url) as f:
        content = f.read()
    return content.decode()


def parse_uri_schemes(content: str) -> list[str]:
    log.info("Parse uri schemes")
    schemes: list[str] = []

    reader = csv.reader(io.StringIO(content), delimiter=",")
    next(reader)  # Skip header row
    for line in reader:
        scheme = line[0].lower().removesuffix(" (obsolete)")
        if not re.fullmatch("[a-z0-9+.-]+", scheme):
            log.warning("unexpected scheme field contents: %s", scheme)
            continue

        schemes.append(scheme)

    return schemes


def generate_output(schemes: list[str], outpath: Path) -> None:
    log.info("Generate output")
    current_date = datetime.now(tz=UTC).isoformat()
    content = BOILER_PLATE.format(current_date=current_date)

    content += "URI_SCHEMES = {\n"
    for scheme in schemes:
        content += f"    '{scheme}',\n"
    content += "}\n"

    outpath.write_text(content)
    log.info("Wrote file to %s", outpath)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Generate IANA data")
    parser.add_argument("out", help="Path to iana module")
    args = parser.parse_args()

    outpath = Path(args.out)
    if not outpath.is_dir():
        sys.exit("Output path is not a directory")

    scheme_content = download_file(SCHEMES_URL)
    schemes = parse_uri_schemes(scheme_content)

    generate_output(schemes, outpath / "uri_schemes.py")
    log.info("Finished !")



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import cast

import logging
from pathlib import Path

from gi.repository import Adw
from gi.repository import Gdk
from gi.repository import GLib
from gi.repository import Gtk

from gajim.common import app
from gajim.common.const import CSSPriority
from gajim.common.settings import Settings

from gajim.gtk.util.icons import get_icon_theme
from gajim.gtk.window import GajimAppWindow


def get_gajim_dir() -> Path:
    gajim_path = Path(__file__) / ".." / ".." / ".." / "gajim"
    return gajim_path.resolve()


def load_style(filename: str, priority: CSSPriority) -> None:
    path = get_gajim_dir() / "data" / "style" / filename
    try:
        with open(str(path), encoding="utf8") as file:
            css = file.read()
    except Exception:
        logging.exception("")  # noqa: LOG015
        return
    provider = Gtk.CssProvider()
    provider.load_from_bytes(GLib.Bytes.new(css.encode("utf-8")))
    display = Gdk.Display.get_default()
    assert display is not None
    Gtk.StyleContext.add_provider_for_display(display, provider, priority)


def get_content_widget(window: GajimAppWindow) -> Gtk.Widget:
    dialog_host = cast(Gtk.Widget, window.get_child())
    internal_bin = cast(Adw.Bin, dialog_host.get_first_child())
    breakpoint_bin = cast(Adw.BreakpointBin, internal_bin.get_child())
    toolbar_view = cast(Adw.ToolbarView, breakpoint_bin.get_child())
    return cast(Gtk.Widget, toolbar_view.get_content())


def init_settings() -> None:
    app.settings = Settings(in_memory=True)
    app.settings.init()


def run_app(load_default_styles: bool = True, load_custom_icons: bool = True) -> None:
    if load_default_styles:
        load_style("gajim.css", CSSPriority.APPLICATION)

    if load_custom_icons:
        icon_theme = get_icon_theme()
        icon_theme.add_search_path(str(get_gajim_dir() / "data/icons"))

    while Gtk.Window.get_toplevels().get_n_items() > 0:
        GLib.MainContext().default().iteration(True)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common import app  # Avoids circular imports from common.helpers
from gajim.common import styling
from gajim.common.styling import EmphasisSpan
from gajim.common.styling import Hyperlink
from gajim.common.styling import PlainBlock
from gajim.common.styling import PreBlock
from gajim.common.styling import PreTextSpan
from gajim.common.styling import process_uris
from gajim.common.styling import QuoteBlock
from gajim.common.styling import StrikeSpan
from gajim.common.styling import StrongSpan
from gajim.common.util.text import escape_iri_query

app.settings.set("additional_uri_schemes", "a a- a. scheme")


STYLING = {
    "pre cannot have children": {
        "input": "_no pre `with *children*`_",
        "tokens": [
            PlainBlock(
                start=0,
                end=26,
                text="_no pre `with *children*`_",
                spans=[
                    PreTextSpan(
                        start=8,
                        start_byte=8,
                        end=25,
                        end_byte=25,
                        text="`with *children*`",
                    ),
                    EmphasisSpan(
                        start=0,
                        start_byte=0,
                        end=26,
                        end_byte=26,
                        text="_no pre `with *children*`_",
                    ),
                ],
            )
        ],
    },
    "nested spans": {
        "input": "_*~children~*_",
        "tokens": [
            PlainBlock(
                start=0,
                end=14,
                text="_*~children~*_",
                spans=[
                    StrikeSpan(
                        start=2, start_byte=2, end=12, end_byte=12, text="~children~"
                    ),
                    StrongSpan(
                        start=1, start_byte=1, end=13, end_byte=13, text="*~children~*"
                    ),
                    EmphasisSpan(
                        start=0,
                        start_byte=0,
                        end=14,
                        end_byte=14,
                        text="_*~children~*_",
                    ),
                ],
            )
        ],
    },
    "spans": {
        "input": "*strong* _emph_~strike~  `pre`",
        "tokens": [
            PlainBlock(
                start=0,
                end=30,
                text="*strong* _emph_~strike~  `pre`",
                spans=[
                    StrongSpan(
                        start=0, start_byte=0, end=8, end_byte=8, text="*strong*"
                    ),
                    EmphasisSpan(
                        start=9, start_byte=9, end=15, end_byte=15, text="_emph_"
                    ),
                    StrikeSpan(
                        start=15, start_byte=15, end=23, end_byte=23, text="~strike~"
                    ),
                    PreTextSpan(
                        start=25, start_byte=25, end=30, end_byte=30, text="`pre`"
                    ),
                ],
            )
        ],
    },
    "spans lazily match": {
        "input": "*strong*plain*",
        "tokens": [
            PlainBlock(
                start=0,
                end=14,
                text="*strong*plain*",
                spans=[
                    StrongSpan(
                        start=0, start_byte=0, end=8, end_byte=8, text="*strong*"
                    )
                ],
            )
        ],
    },
    "start span only": {
        "input": "*not strong",
        "tokens": [PlainBlock(start=0, end=11, text="*not strong", spans=[])],
    },
    "byte pos is different": {
        "input": "*ö* *öö*",
        "tokens": [
            PlainBlock(
                start=0,
                end=8,
                text="*ö* *öö*",
                spans=[
                    StrongSpan(start=0, start_byte=0, end=3, end_byte=4, text="*ö*"),
                    StrongSpan(start=4, start_byte=5, end=8, end_byte=11, text="*öö*"),
                ],
            )
        ],
    },
    "byte pos is different with multiple blocks": {
        "input": "```\npre\n```\n*pláin*",
        "tokens": [
            PreBlock(start=0, end=12, text="```\npre\n```\n"),
            PlainBlock(
                start=12,
                end=19,
                text="*pláin*",
                spans=[
                    StrongSpan(start=0, start_byte=0, end=7, end_byte=8, text="*pláin*")
                ],
            ),
        ],
    },
    "end span only": {
        "input": "not strong*",
        "tokens": [PlainBlock(start=0, end=11, text="not strong*", spans=[])],
    },
    "invalid end span": {
        "input": "*not *strong",
        "tokens": [PlainBlock(start=0, end=12, text="*not *strong", spans=[])],
    },
    "empty span": {
        "input": "**",
        "tokens": [PlainBlock(start=0, end=2, text="**", spans=[])],
    },
    "3 unmatched directives": {
        "input": "***",
        "tokens": [PlainBlock(start=0, end=3, text="***", spans=[])],
    },
    "4 unmatched directives": {
        "input": "****",
        "tokens": [PlainBlock(start=0, end=4, text="****", spans=[])],
    },
    "invalid diretives ignored": {
        "input": "* plain *strong*",
        "tokens": [
            PlainBlock(
                start=0,
                end=16,
                text="* plain *strong*",
                spans=[
                    StrongSpan(
                        start=8, start_byte=8, end=16, end_byte=16, text="*strong*"
                    )
                ],
            )
        ],
    },
    "uneven start directives": {
        "input": "*this is *uneven*",
        "tokens": [
            PlainBlock(
                start=0,
                end=17,
                text="*this is *uneven*",
                spans=[
                    StrongSpan(
                        start=9, start_byte=9, end=17, end_byte=17, text="*uneven*"
                    )
                ],
            )
        ],
    },
    "overlapping directives": {
        "input": "*this cannot _overlap*_",
        "tokens": [
            PlainBlock(
                start=0,
                end=23,
                text="*this cannot _overlap*_",
                spans=[
                    StrongSpan(
                        start=0,
                        start_byte=0,
                        end=22,
                        end_byte=22,
                        text="*this cannot _overlap*",
                    )
                ],
            )
        ],
    },
    "plain blocks": {
        "input": "one\nand two",
        "tokens": [PlainBlock(start=0, end=11, text="one\nand two", spans=[])],
    },
    "pre block with closing": {
        "input": "```\npre *fmt* ```\n```\nplain",
        "tokens": [
            PreBlock(start=0, end=22, text="```\npre *fmt* ```\n```\n"),
            PlainBlock(start=22, end=27, text="plain", spans=[]),
        ],
    },
    "pre block EOF": {
        "input": "````\na\n```",
        "tokens": [PreBlock(start=0, end=10, text="````\na\n```")],
    },
    "pre block no terminator EOF": {
        "input": "```\na```",
        "tokens": [PlainBlock(start=0, end=8, text="```\na```", spans=[])],
    },
    "pre block no body EOF": {
        "input": "```newtoken\n",
        "tokens": [PlainBlock(start=0, end=12, text="```newtoken\n", spans=[])],
    },
    "single level block quote": {
        "input": ">  quoted\nnot quoted",
        "tokens": [
            QuoteBlock(
                start=0,
                end=10,
                text=">  quoted\n",
                blocks=[PlainBlock(start=0, end=8, text=" quoted\n", spans=[])],
            ),
            PlainBlock(start=10, end=20, text="not quoted", spans=[]),
        ],
    },
    "multi level block quote": {
        "input": ">  quoted\n>>   quote > 2\n>quote 1\n\nnot quoted",
        "tokens": [
            QuoteBlock(
                start=0,
                end=34,
                text=">  quoted\n>>   quote > 2\n>quote 1\n",
                blocks=[
                    PlainBlock(start=0, end=8, text=" quoted\n", spans=[]),
                    QuoteBlock(
                        start=8,
                        end=22,
                        text=">   quote > 2\n",
                        blocks=[
                            PlainBlock(start=0, end=12, text="  quote > 2\n", spans=[])
                        ],
                    ),
                    PlainBlock(start=22, end=30, text="quote 1\n", spans=[]),
                ],
            ),
            PlainBlock(start=34, end=45, text="\nnot quoted", spans=[]),
        ],
    },
    "quote start then EOF": {
        "input": "> ",
        "tokens": [QuoteBlock(start=0, end=2, text="> ", blocks=[])],
    },
    "quote with children": {
        "input": "> ```\n> pre\n> ```\n> not pre",
        "tokens": [
            QuoteBlock(
                start=0,
                end=27,
                text="> ```\n> pre\n> ```\n> not pre",
                blocks=[
                    PreBlock(start=0, end=12, text="```\npre\n```\n"),
                    PlainBlock(start=12, end=19, text="not pre", spans=[]),
                ],
            )
        ],
    },
    "pre end of parent": {
        "input": "> ``` \n> pre\nplain",
        "tokens": [
            QuoteBlock(
                start=0,
                end=13,
                text="> ``` \n> pre\n",
                blocks=[PreBlock(start=0, end=9, text="``` \npre\n")],
            ),
            PlainBlock(start=13, end=18, text="plain", spans=[]),
        ],
    },
    "span lines": {
        "input": "*not \n strong*",
        "tokens": [PlainBlock(start=0, end=14, text="*not \n strong*", spans=[])],
    },
    "plain with uri": {
        "input": "some kind of link http://foo.com/blah_blah",
        "tokens": [
            PlainBlock(
                start=0,
                end=42,
                text="some kind of link http://foo.com/blah_blah",
                spans=[],
                uris=[
                    Hyperlink(
                        start=18,
                        start_byte=18,
                        end=42,
                        end_byte=42,
                        text="http://foo.com/blah_blah",
                        uri="http://foo.com/blah_blah",
                    )
                ],
            )
        ],
    },
    "plain with uri don’t consider comma": {
        "input": "some kind of link http://foo.com/blah_blah,",
        "tokens": [
            PlainBlock(
                start=0,
                end=43,
                text="some kind of link http://foo.com/blah_blah,",
                spans=[],
                uris=[
                    Hyperlink(
                        start=18,
                        start_byte=18,
                        end=42,
                        end_byte=42,
                        text="http://foo.com/blah_blah",
                        uri="http://foo.com/blah_blah",
                    )
                ],
            )
        ],
    },
    "plain with uri and styling": {
        "input": "some *kind* of link http://foo.com/blah_blah",
        "tokens": [
            PlainBlock(
                start=0,
                end=44,
                text="some *kind* of link http://foo.com/blah_blah",
                spans=[
                    StrongSpan(
                        start=5, start_byte=5, end=11, end_byte=11, text="*kind*"
                    )
                ],
                uris=[
                    Hyperlink(
                        start=20,
                        start_byte=20,
                        end=44,
                        end_byte=44,
                        text="http://foo.com/blah_blah",
                        uri="http://foo.com/blah_blah",
                    )
                ],
            )
        ],
    },
    "plain with multiple uris": {
        "input": "some http://foo.com/blah_blah and http://foo.com/blah_blah/123",
        "tokens": [
            PlainBlock(
                start=0,
                end=62,
                text="some http://foo.com/blah_blah and http://foo.com/blah_blah/123",
                spans=[],
                uris=[
                    Hyperlink(
                        start=5,
                        start_byte=5,
                        end=29,
                        end_byte=29,
                        text="http://foo.com/blah_blah",
                        uri="http://foo.com/blah_blah",
                    ),
                    Hyperlink(
                        start=34,
                        start_byte=34,
                        end=62,
                        end_byte=62,
                        text="http://foo.com/blah_blah/123",
                        uri="http://foo.com/blah_blah/123",
                    ),
                ],
            )
        ],
    },
}


# Most of the URI/JID test sets belong in test_regex.py, and should be imported
# here somehow (TODO).
URIS = [
    "a:b",
    "a-:b",
    "a.:b",
    "xmpp:conference.gajim.org",
    "xmpp:asd@at",
    "xmpp:asd@asd.at",
    "xmpp:asd-asd@asd.asdasd.at.",
    # 'xmpp:me@%5B::1%5D', FIXME
    "xmpp:myself@127.13.42.69",
    "xmpp:myself@127.13.42.69/localhost",
    "xmpp:%23room%25irc.example@biboumi.xmpp.example",
    "xmpp:+15551234567@cheogram.com",
    "xmpp:romeo@montague.net?message;subject=Test%20Message;body=Here%27s%20a%20test%20message",
    "geo:1,2",
    "geo:1,2,3",
    "file:/foo/bar/baz",  # xffm
    "file:///foo/bar/baz",  # nautilus, rox
    "file:///x:/foo/bar/baz",  # windows
    "file://localhost/foo/bar/baz",
    "file://nonlocalhost/foo/bar/baz",
    # These seem to be from https://mathiasbynens.be/demo/url-regex
    "http://foo.com/blah_blah",
    "http://foo.com/blah_blah/",
    "http://foo.com/blah_blah_(wikipedia)",
    "http://foo.com/blah_blah_(wikipedia)_(again)",
    "http://www.example.com/wpstyle/?p=364",
    "https://www.example.com/foo/?bar=baz&inga=42&quux",
    "https://www.example.com?foo=%s",
    "http://✪df.ws/123",
    "http://userid:password@example.com:8080",
    "http://userid:password@example.com:8080/",
    "http://userid@example.com",
    "http://userid@example.com/",
    "http://userid@example.com:8080",
    "http://userid@example.com:8080/",
    "http://userid:password@example.com",
    "http://userid:password@example.com/",
    "http://142.42.1.1/",
    "http://142.42.1.1:8080/",
    "http://➡.ws/䨹",
    "http://⌘.ws",
    "http://⌘.ws/",
    "http://foo.com/blah_(wikipedia)#cite-1",
    "http://foo.com/blah_(wikipedia)_blah#cite-1",
    "http://foo.com/unicode_(✪)_in_parens",
    "http://foo.com/(something)?after=parens",
    "http://☺.damowmow.com/",
    "http://code.google.com/events/#&product=browser",
    "http://j.mp",
    "ftp://foo.bar/baz",
    "http://foo.bar/?q=Test%20URL-encoded%20stuff",
    "http://مثال.إختبار",
    "http://例子.测试",
    "http://उदाहरण.परीक्षा",
    "http://-.~_!$&'()*+,;=:%40:80%2f::::::@example.com",
    "http://1337.net",
    "http://a.b-c.de",
    "http://223.255.255.254",
    "https://foo_bar.example.com/",
    # These are from https://rfc-editor.org/rfc/rfc3513#section-2.2
    "http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]",
    "http://[1080:0:0:0:8:800:200C:417A]",
    "http://[1080:0:0:0:8:800:200C:417A]",
    "http://[FF01:0:0:0:0:0:0:101]",
    "http://[0:0:0:0:0:0:0:1]",
    "http://[0:0:0:0:0:0:0:0]",
    "http://[1080::8:800:200C:417A]",
    "http://[FF01::101]",
    "http://[::1]",
    "http://[::]",
    "http://[0:0:0:0:0:0:13.1.68.3]",
    "http://[0:0:0:0:0:FFFF:129.144.52.38]",
    "http://[::13.1.68.3]",
    "http://[::FFFF:129.144.52.38]",
    # These are from https://rfc-editor.org/rfc/rfc3986#section-1.1.2
    "ftp://ftp.is.co.za/rfc/rfc1808.txt",
    "http://www.ietf.org/rfc/rfc2396.txt",
    "ldap://[2001:db8::7]/c=GB?objectClass?one",
    "mailto:John.Doe@example.com",
    "news:comp.infosystems.www.servers.unix",
    "tel:+1-816-555-1212",
    "telnet://192.0.2.16:80/",
    "urn:oasis:names:specification:docbook:dtd:xml:4.1.2",
]


# * non-URI foos
# * non-absolute URIs
NONURIS = [
    "",
    " ",
    ".",
    ":",
    "://",
    "Böblingen:🌥",
    "Bo\u0308blingen:🌥",  # finds blingen:🌥 but rejects an unregistered scheme
    "path/to/file_na.me:123",
    "_sche.me:body_",
    "_sche+me:body_",
    ".scheme:body",
    "+scheme:body",
    # These are from https://mathiasbynens.be/demo/url-regex
    "//",
    "//a",
    "///a",
    "///",
    "foo.com",
]


# * valid scheme-only URIs
# * valid generic URIs that fail requirements of their specific scheme.
UNACCEPTABLE_URIS = [
    "scheme:",
    # These are from https://mathiasbynens.be/demo/url-regex
    "http://",
    "http://?",
    "http://??",
    "http://??/",
    "http://#",
    "http://##",
    "http://##/",
    "http:///a",
    "geo:1,",
    "geo:,2",
    # 'geo:1,2,',  FIXME: wrongly parsed as valid
    "geo:1,,3",
    "geo:,2,3",
    "geo:1,,",
    "geo:,2,",
    "geo:,,3",
    "geo:,,",
    "file:",
    "file:a",
    "file:a/",
    "file:a/b",
    "mailtomailto:foo@bar.com.uk",
]


JIDS = [
    "asd@at",
    "asd@asd.at",
    "asd@asd.asd.at",
    "asd@asd.asd-asd.at",
    "asd.asd@asd.asd-asd.at",
    "asd-asd@asd.asdasd.at",
    "asd-asd@asd.asdasd.at.",
    "me@[::1]",
    "myself@127.13.42.69",
    "#room%irc.example@biboumi.xmpp.example",
    "+15551234567@cheogram.com",
    # These are from https://rfc-editor.org/rfc/rfc7622#section-3.5
    "fußball@example.com",
    "π@example.com",
    # These are from https://xmpp.org/extensions/xep-0106.html#examples
    r"space\20cadet@example.com",
    r"call\20me\20\22ishmael\22@example.com",
    r"at\26t\20guy@example.com",
    r"d\27artagnan@example.com",
    r"\2f.fanboy@example.com",
    r"\3a\3afoo\3a\3a@example.com",
    r"\3cfoo\3e@example.com",
    r"user\40host@example.com",
    r"c\3a\net@example.com",
    r"c\3a\\net@example.com",
    r"c\3a\cool\20stuff@example.com",
    r"c\3a\5c5commas@example.com",
    r"here\27s_a_wild_\26_\2fcr%zy\2f_address@example.com",
    r"here\27s_a_wild_\26_\2fcr%zy\2f_address_for\3a\3cwv\3e(\22IMPS\22)@example.com",
    # Some more from the same document
    r"tréville\40musketeers.lit@smtp.gascon.fr",
    r"\5c3and\2is\5c5cool@example.com",
    r"CN=D\27Artagnan\20Saint-Andr\E9,O=Example\20\26\20Company,\20Inc.,DC=example,DC=com@st.example.com",
    r"somenick!user\22\26\27\2f\3a\3c\3e\5c3address@example.com",
    # https://en.wikipedia.org/wiki/E-mail_address#Internationalization_examples
    # Do note that these are *e-mail* addresses and might not all be valid JIDs.
    "Pelé@example.com",
    "δοκιμή@παράδειγμα.δοκιμή",
    "我買@屋企.香港",
    "二ノ宮@黒川.日本",
    "медведь@с-балалайкой.рф",
    # 'संपर्क@डाटामेल.भारत',  fails because of the 2 combining chars in localpart
]

NONJIDS = [
    "",
    "@",
    # These are from https://rfc-editor.org/rfc/rfc7622#section-3.5
    '"juliet"@example.com',
    "@example.com",
    "henryⅣ@example.com",  # localpart has a compatibility-decomposable cp
    "♚@example.com",  # localpart has a symbol cp
    "juliet@",
]


URIS_WITH_TEXT = [
    (
        "write to my email mailto:foo@bar.com.uk (but not to mailto:bar@foo.com)",
        ["mailto:foo@bar.com.uk", "mailto:bar@foo.com"],
    ),
    ("see this http://userid@example.com/ link", ["http://userid@example.com/"]),
    ("see this http://userid@example.com/, and ..", ["http://userid@example.com/"]),
    ("<http://userid@example.com/>", ["http://userid@example.com/"]),
    ('"http://userid@example.com/"', ["http://userid@example.com/"]),
    (
        "regexes are useless (see https://en.wikipedia.org/wiki/Recursion_(computer_science)), but comfy",
        ["https://en.wikipedia.org/wiki/Recursion_(computer_science)"],
    ),
    ("(scheme:body)", ["scheme:body"]),
    ("/scheme:body", ["scheme:body"]),
    ("!scheme:body", ["scheme:body"]),
]


class Test(unittest.TestCase):
    @staticmethod
    def wrap(link: str) -> str:
        return f"Prologue (link: {link}), and epilogue!"

    def test_styling(self):
        for params in STYLING.values():
            assert isinstance(params["input"], str)
            result = styling.process(params["input"])
            self.assertEqual(result.blocks, params["tokens"])

    def test_uris(self):
        for uri in URIS:
            text = self.wrap(uri)
            hlinks = process_uris(text)
            self.assertEqual([link.uri for link in hlinks], [uri], text)

    def test_invalid_uris(self):
        for foo in NONURIS + UNACCEPTABLE_URIS:
            text = self.wrap(foo)
            hlinks = process_uris(text)
            self.assertEqual([link.text for link in hlinks], [], text)

    def test_jids(self):
        for jidlike in JIDS:
            text = self.wrap(jidlike)
            uri = escape_iri_query(jidlike)
            hlinks = process_uris(text)
            self.assertEqual(
                [(link.text, link.uri) for link in hlinks], [(jidlike, uri)], text
            )

    def test_nonjids(self):
        for foo in NONJIDS:
            text = self.wrap(foo)
            hlinks = process_uris(text)
            self.assertEqual([link.text for link in hlinks], [], text)

    def test_uris_with_text(self):
        for text, results in URIS_WITH_TEXT:
            hlinks = process_uris(text)
            self.assertEqual([link.text for link in hlinks], results, text)


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
import unittest
from unittest.mock import patch

from gajim.common.helpers import sanitize_filename


class SanitizeTest(unittest.TestCase):
    """Tests for the sanitize_filename function."""

    @patch.object(sys, "platform", "win32")
    def test_invalid_chars(self):
        """Make sure invalid characters are removed in filenames"""
        self.assertEqual(sanitize_filename("A/B/C"), "ABC")
        self.assertEqual(sanitize_filename("A*C.d"), "AC.d")
        self.assertEqual(sanitize_filename("A?C.d"), "AC.d")

    @patch.object(sys, "platform", "win32")
    def test_invalid_suffix(self):
        """Dots are not allowed at the end"""
        self.assertEqual(sanitize_filename("def."), "def")
        self.assertEqual(sanitize_filename("def.ghi"), "def.ghi")
        self.assertTrue(sanitize_filename("X" * 1000 + ".").endswith("X"))

    @patch.object(sys, "platform", "win32")
    def test_reserved_words(self):
        """Make sure reserved Windows words are prefixed"""
        self.assertEqual(sanitize_filename("NUL"), "__NUL")
        self.assertEqual(sanitize_filename(".."), "__")

    @patch.object(sys, "platform", "win32")
    def test_long_names(self):
        """Make sure long names are truncated"""
        self.assertEqual(len(sanitize_filename("X" * 300)), 50)
        self.assertEqual(
            len(sanitize_filename(".".join(["X" * 100, "X" * 100, "X" * 100]))), 50
        )
        self.assertEqual(
            len(sanitize_filename(".".join(["X" * 300, "X" * 300, "X" * 300]))), 50
        )
        self.assertEqual(len(sanitize_filename("." * 300 + ".txt")), 50)

    @patch.object(sys, "platform", "win32")
    def test_unicode_normalization(self):
        """Names should be NFKD normalized"""
        self.assertEqual(sanitize_filename("ў"), chr(1091) + chr(774))

    @patch.object(sys, "platform", "win32")
    def test_extensions(self):
        """Filename extensions should be preserved when possible."""
        really_long_name = "X" * 1000 + ".pdf"
        self.assertTrue(sanitize_filename(really_long_name).endswith(".pdf"))
        self.assertTrue(sanitize_filename("X" * 1000).endswith("X"))
        self.assertTrue(
            sanitize_filename("X" * 100 + "." + "X" * 100 + ".pdf").endswith(".pdf")
        )
        self.assertTrue(sanitize_filename("X" * 100 + "." + "X" * 400).endswith("X"))
        self.assertTrue(
            sanitize_filename("X" * 100 + "." + "X" * 400 + ".pdf").endswith(".pdf")
        )

    def test_right_to_left_codepoint(self):
        file_name = "test_‮_fdp.exe"  # noqa: PLE2502
        self.assertEqual(sanitize_filename(file_name), "test__fdp.exe")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import GLib
from gi.repository import Gtk

from gajim.gtk.apply_button_box import ApplyButtonBox
from gajim.gtk.window import GajimAppWindow

from . import util


class TestApplyButtonBox(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True)
        self.set_child(box)

        self._apply_button_box = ApplyButtonBox("Apply", self._on_clicked)
        self._apply_button_box.set_button_state(True)
        box.append(self._apply_button_box)

    def _on_clicked(self, _button: Gtk.Button) -> None:
        print("clicked")
        GLib.timeout_add_seconds(2, self._apply_button_box.set_success)
        GLib.timeout_add_seconds(
            5, self._apply_button_box.set_error, "An error occurred"
        )
        GLib.timeout_add_seconds(9, self._apply_button_box.set_button_state, True)
        print("Finished")


util.init_settings()

window = TestApplyButtonBox()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.storage import MessageArchiveStorage


class ThreadsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def _create_base_message(self, message_id: str, thread_id: str) -> Message:
        return Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id=message_id,
            stanza_id=get_uuid(),
            thread_id_=thread_id,
            text="message",
        )

    def test_insert_thread(self) -> None:
        # Insert thread and join afterwards

        message_data = self._create_base_message(message_id="1", thread_id="t1")
        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.thread is not None

        self.assertEqual(message.thread.pk, 1)
        self.assertEqual(message.thread.fk_account_pk, 1)
        self.assertEqual(message.thread.fk_remote_pk, 1)
        self.assertEqual(message.thread.id, "t1")

        # Same thread yields same thread pk

        message_data = self._create_base_message(message_id="2", thread_id="t1")
        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.thread is not None

        self.assertEqual(message.thread.pk, 1)
        self.assertEqual(message.thread.fk_account_pk, 1)
        self.assertEqual(message.thread.fk_remote_pk, 1)
        self.assertEqual(message.thread.id, "t1")

        # Different thread yield different thread pk

        message_data = self._create_base_message(message_id="3", thread_id="t2")
        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.thread is not None

        self.assertEqual(message.thread.pk, 2)
        self.assertEqual(message.thread.fk_account_pk, 1)
        self.assertEqual(message.thread.fk_remote_pk, 1)
        self.assertEqual(message.thread.id, "t2")

        # Different remote jid with same thread id yields new thread pk

        message_data = self._create_base_message(message_id="4", thread_id="t2")
        message_data.remote_jid_ = JID.from_string("remote2@jid.org")
        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.thread is not None

        self.assertEqual(message.thread.pk, 3)
        self.assertEqual(message.thread.fk_account_pk, 1)
        self.assertEqual(message.thread.fk_remote_pk, 2)
        self.assertEqual(message.thread.id, "t2")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Adw
from gi.repository import Gtk

from gajim.gtk.sidebar_switcher import SideBarMenuItem
from gajim.gtk.sidebar_switcher import SideBarSwitcher
from gajim.gtk.window import GajimAppWindow

from . import util

PAGES = ["A", "B", "B1", "B2", "B3", "C", "D", "E", "E1", "E2", "E3", "F"]


class TestSideBarSwitcher(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
        )

        stack = Gtk.Stack()
        for p in PAGES:
            stack.add_named(Gtk.Label(label=p), p)

        side_bar_switcher = SideBarSwitcher()
        side_bar_switcher.set_with_menu(
            stack,
            [
                SideBarMenuItem("A", "A", icon_name="lucide-laptop-symbolic"),
                SideBarMenuItem(
                    "B",
                    "B",
                    icon_name="user-available-symbolic",
                    children=[
                        SideBarMenuItem("B1", "B1", icon_name="lucide-laptop-symbolic"),
                        SideBarMenuItem("B2", "B2", icon_name="lucide-laptop-symbolic"),
                        SideBarMenuItem("B3", "B3", icon_name="lucide-laptop-symbolic"),
                    ],
                ),
                SideBarMenuItem(
                    "C",
                    "C",
                    group="Group 1",
                    icon_name="lucide-message-circle-symbolic",
                ),
                SideBarMenuItem(
                    "D",
                    "D",
                    group="Group 1",
                    icon_name="lucide-mic-symbolic",
                ),
                SideBarMenuItem(
                    "E",
                    "E",
                    group="Group 2",
                    icon_name="lucide-megaphone-symbolic",
                    children=[
                        SideBarMenuItem("E1", "E1", icon_name="lucide-laptop-symbolic"),
                        SideBarMenuItem("E2", "E2", icon_name="lucide-laptop-symbolic"),
                        SideBarMenuItem("E3", "E3", icon_name="lucide-laptop-symbolic"),
                    ],
                ),
                SideBarMenuItem("F", "F", icon_name="lucide-palette-symbolic"),
            ],
        )

        toolbar = Adw.ToolbarView(content=side_bar_switcher)
        toolbar.add_top_bar(Adw.HeaderBar())

        sidebar_page = Adw.NavigationPage(
            title="Preferences", tag="sidebar", child=toolbar
        )

        toolbar = Adw.ToolbarView(content=stack)
        toolbar.add_top_bar(Adw.HeaderBar())

        content_page = Adw.NavigationPage(title=" ", tag="content", child=toolbar)

        nav = Adw.NavigationSplitView(sidebar=sidebar_page, content=content_page)

        self.set_child(nav)

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestSideBarSwitcher()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

import os
from pathlib import Path

from gi.repository import Adw
from gi.repository import Gtk

from gajim.gtk.window import GajimAppWindow

from . import util

CUSTOM_ICONS_PATH = util.get_gajim_dir() / "data/icons"


class TestCustomIcons(GajimAppWindow):
    """A test window which lists all of Gajim's custom scalable icons.
    This allows us to quickly check if symbolic display is working correctly, see:
    https://dev.gajim.org/gajim/gajim/-/wikis/Icon-Resources#symbolic-svgs
    """

    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(spacing=18, orientation=Gtk.Orientation.VERTICAL)
        self.set_child(box)

        switch = Gtk.Switch(valign=Gtk.Align.CENTER)
        switch.connect("notify::active", self._on_theme_switched)

        reload_button = Gtk.Button(label="Reload")
        reload_button.connect("clicked", self._on_reload_clicked)

        controls_box = Gtk.Box(spacing=12, halign=Gtk.Align.CENTER)
        controls_box.append(Gtk.Label(label="Light"))
        controls_box.append(switch)
        controls_box.append(Gtk.Label(label="dark"))
        controls_box.append(reload_button)
        box.append(controls_box)

        self._adw_style_manager = Adw.StyleManager.get_default()
        color_scheme = self._adw_style_manager.get_color_scheme()
        switch.set_active(color_scheme == Adw.ColorScheme.FORCE_DARK)

        scrolled_window = Gtk.ScrolledWindow(
            hscrollbar_policy=Gtk.PolicyType.NEVER, vexpand=True
        )
        box.append(scrolled_window)
        self._flow_box = Gtk.FlowBox(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            row_spacing=6,
            column_spacing=12,
            min_children_per_line=10,
        )
        scrolled_window.set_child(self._flow_box)

        self._load_images()

    def _cleanup(self):
        pass

    def _on_theme_switched(self, switch: Gtk.Switch, args: Any) -> None:
        if switch.get_active():
            self._adw_style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
        else:
            self._adw_style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)

    def _on_reload_clicked(self, _button: Gtk.Button) -> None:
        self._load_images()

    def _load_images(self) -> None:
        self._flow_box.remove_all()

        for _root, _dirs, files in os.walk(CUSTOM_ICONS_PATH):
            for file in sorted(files):
                file_path = Path(file)

                if not file_path.suffix == ".svg":
                    continue

                image = Gtk.Image.new_from_icon_name(file_path.stem)
                image.set_tooltip_text(file)
                image.set_pixel_size(48)
                self._flow_box.append(image)


util.init_settings()

window = TestCustomIcons()
window.show()

util.run_app()
